/*

	jackmixdesk_gtk.c
	Copyright (C) 2007  Ulrich Lorenz Schlueter

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <unistd.h>
#include <signal.h>
#include <getopt.h>
#include <stringprep.h>

#include <gtk/gtk.h>

#include <jack/jack.h>

#include <lo/lo.h>
#include <lash/lash.h>

#include <libxml/xmlwriter.h>
#include <libxml/parser.h>
#include <libxml/encoding.h>

#include "config.h"

#include "phatfanslider.h"
#include "phatvfanslider.h"
#include "phathfanslider.h"
#include "phatkeyboard.h"
#include "phatvkeyboard.h"
#include "phathkeyboard.h"
#include "phatpad.h"
#include "phatknob.h"

#define		DEFAULT_CLIENT_NAME		"JackMixDesk_gtk"		/* Default name of client */
#define		DEFAULT_MIXER_PORT		"16000"					/* Default jackmixdesk port */
#define		DEFAULT_MIXER_HOST		"localhost"				/* Default jackmixdesk host */
#define		DEFAULT_LABEL_LEN		(32)					/* Max length of label strings */
#define		DEFAULT_ENTRY_LEN		(4)						/* Max length of entry */
#define		DEFAULT_ENTRY_DIGITS	(2)						/* entry digits */
#define		KNOB_GAIN_MIN			(-90)					/* Min range for gain knobs */
#define		KNOB_GAIN_MAX			(0)						/* Max range for gain knobs */
#define		KNOB_GAIN_STEP			(0.05)					/* Step width for gain knobs */
#define		KNOB_PAN_MIN			(-90)					/* Min range for pan knobs */
#define		KNOB_PAN_MAX			(90)					/* Max range for pan knobs */
#define		KNOB_PAN_STEP			(0.05)					/* Step width for pan knobs */
#define		SPINBUTTON_FONT_SIZE	(7679)					/* Font size for spin buttons */
#define		SMALL_LABEL_FONT_SIZE	(8700)					/* Font size for small labels */
#define		MEDIUM_LABEL_FONT_SIZE	(8750)					/* Font size for bigger labels */
#define		KNOB_SIZE				(32)					/* Default knob size */
#define		BUTTON_SIZE				(10)					/* Default button size */
#define		BOX_HSPACING			(3)						/* Default box spacing */
#define		BOX_VSPACING			(2)						/* Default box spacing */
#define		INPUT_WIDTH				(72)					/* Intput channel width */
#define		OUTPUT_WIDTH			(50)					/* Output channel width */
#define		SEPARATOR_WIDTH			(2)						/* Channel separator width */
#define		CHANNEL_OFFSET			(23)
#define		OUTPUT_OFFSET			(4)
#define		TIMEOUT					(5)						/* Time to wait for the mixer */
#define 	DEFAULT_CONFIG_FILE		".mixdeskgtkrc"			/* Default config file name */
#define 	DEFAULT_CONFIG_PATH		"."						/* Default config file path */
#define 	DEFAULT_XML_ENCODING	"UTF-8"					/* Default xml encoding */
#define 	DEFAULT_MESSAGE_WIDTH	(280)					/* gtk message boxes size */
#define 	DEFAULT_MESSAGE_HEIGHT	(112)					/* gtk message boxes size */


typedef struct {
	gfloat gain;						/* decibels */
	gfloat pan;
	gfloat gain_l;						/* decibels */
	gfloat gain_r;						/* decibels */

	GtkWidget *label_send;
	GtkWidget *box_vbox;
	GtkWidget *box_hbox_upper;
	GtkWidget *box_hbox_middle;
	GtkWidget *box_hbox_lower;
	GtkWidget *box_vbox_middle_left;
	GtkWidget *box_vbox_middle_right;
	GtkWidget *box_hbox_middle_knob_left;
	GtkWidget *box_hbox_middle_knob_right;
	GtkWidget *box_hbox_middle_entry_left;
	GtkWidget *box_hbox_middle_entry_right;
	GtkWidget *box_vbox_lower_left;
	GtkWidget *box_vbox_lower_right;
	GtkWidget *box_hbox_lower_knob_left;
	GtkWidget *box_hbox_lower_knob_right;
	GtkWidget *box_hbox_lower_entry_left;
	GtkWidget *box_hbox_lower_entry_right;
	GtkWidget *label_out_top;
	GtkWidget *label_pan;
	GtkWidget *label_gain_l;
	GtkWidget *label_gain_r;
	GtkWidget *knob_gain;
	GtkWidget *knob_pan;
	GtkWidget *knob_gain_l;
	GtkWidget *knob_gain_r;
	GtkWidget *spinbutton_gain;
	GtkWidget *spinbutton_pan;
	GtkWidget *spinbutton_gain_l;
	GtkWidget *spinbutton_gain_r;
	GtkWidget *separator;
} jmd_send_t;

typedef struct {
	gchar label[DEFAULT_LABEL_LEN];	/* Label for Channel */
	gint button_stereo_state;
	gint button_mute_state;

	jmd_send_t *send;				/* Aux Sends */

	GtkWidget *box_vbox;
	GtkWidget *box_hbox_knob_pan;
	GtkWidget *box_hbox_entry_pan;
	GtkWidget *box_hbox_entry_gain;
	GtkWidget *box_hbox_button_stereo;
	GtkWidget *box_hbox_button_mute;
	GtkWidget *label_gain;
	GtkWidget *label_pan;
	GtkWidget *knob_pan;			/* Same as first send gain */
	GtkWidget *fader_gain;			/* Same as first send gain */
	GtkWidget *spinbutton_gain;
	GtkWidget *spinbutton_pan;
	GtkWidget *spacer_stereo;
	GtkWidget *label_stereo;
	GtkWidget *button_stereo;
	GtkWidget *spacer_mute;
	GtkWidget *button_mute;
	GtkWidget *label_mute;
	GtkWidget *label_channel_top;
	GtkWidget *label_channel_out;
	GtkWidget *label_channel_bottom;
	GtkWidget *separator;
	GtkWidget *separator_top;
	GtkWidget *separator_buttons;
	GtkWidget *separator_bottom;
} jmd_channel_t;

typedef struct {
	gchar label[DEFAULT_LABEL_LEN];	/* Label for Output */
	gfloat gain;					/* decibels */
	gint button_mute_state;
	gint button_link_state;

	GtkWidget *box_vbox;
	GtkWidget *separator;
	GtkWidget *separator_top;
	GtkWidget *separator_bottom;
	GtkWidget *separator_buttons;
	GtkWidget *box_hbox_label_top;
	GtkWidget *box_hbox_label_bottom;
	GtkWidget *box_hbox_fader;
	GtkWidget *box_hbox_entry;
	GtkWidget *label_out_top;
	GtkWidget *label_gain;
	GtkWidget *label_out_bottom;
	GtkWidget *fader_gain;
	GtkWidget *spinbutton_gain;
	GtkWidget *box_hbox_mute;
	GtkWidget *spacer_mute;
	GtkWidget *button_mute;
	GtkWidget *label_mute;
	GtkWidget *box_hbox_link;
	GtkWidget *spacer_link;
	GtkWidget *label_link;
	GtkWidget *button_link;
} jmd_output_t;

typedef struct {
	GtkWidget *vbox;

	GtkWidget *hbox_controls;
	GtkWidget *hbox_keyboard;
	GtkWidget *scroll;
	GtkWidget *keyboard;
} jmd_keyboard_t;

typedef struct {
	GtkWidget *vbox;

	GtkWidget *hbox_controls;
	GtkWidget *hbox_phatpad;
	GtkWidget *scroll;
	GtkWidget *phatpad;
} jmd_phatpad_t;

typedef struct {
	GtkWidget *box_vbox;
	GtkWidget *separator;
	GtkWidget *separator_top;
	GtkWidget *separator_bottom;
	GtkWidget *separator_buttons;
	GtkWidget *box_hbox_label_top;
	GtkWidget *box_hbox_label_bottom;
	GtkWidget *box_hbox_fader;
	GtkWidget *box_hbox_entry;
	GtkWidget *label_out_top;
	GtkWidget *label_gain;
	GtkWidget *label_out_bottom;
	GtkWidget *fader_volume;
	GtkWidget *spinbutton_volume;
} jmd_midimix_channel_t;

typedef struct {
	gint channel_count;
	GtkWidget *window_top;
    GtkWidget *window_midimixer;
    GtkWidget *box_channels_hbox;

    jmd_midimix_channel_t *channel;
    jmd_keyboard_t **keyboard;
} jmd_midimix_t;

typedef struct {
	gint channel_count;
	GtkWidget *window_top;
    GtkWidget *window_tools;
    GtkWidget *box_channels_hbox;

    jmd_midimix_t *midimix;
    jmd_keyboard_t *keyboard;
    jmd_phatpad_t *phatpad;
} jmd_tools_t;

typedef struct {
	gchar label[DEFAULT_LABEL_LEN];	/* Label for Port */
	jack_port_t *port;
} jmd_midiport_t;

typedef struct {
	gboolean keep_above;

	GtkWidget *window;
	GtkWidget *box_vbox;

	GtkWidget *menu_bar;

	GtkWidget *file_menu;
	GtkWidget *file_item;
    GtkWidget *close_item;
    GtkWidget *quit_item;

	GtkWidget *edit_menu;
	GtkWidget *edit_item;
	GtkWidget *add_port_menu;
	GtkWidget *add_port_item;
	GtkWidget *add_in_port_item;
	GtkWidget *add_out_port_item;
	GtkWidget *del_port_menu;
	GtkWidget *del_port_item;
	GtkWidget *del_in_port_item;
	GtkWidget *del_out_port_item;

	GtkWidget *view_menu;
	GtkWidget *view_item;
	GtkWidget *midimix_item;
	GtkWidget *keyboard_item;
	GtkWidget *phatpad_item;

	GtkWidget *window_menu;
	GtkWidget *window_item;
	GtkWidget *window_tearoff_item;
	GtkWidget *channel_show_item;
	GtkWidget *output_show_item;
	GtkWidget *menu_above_separator;
	GtkWidget *menu_above_item;

	GtkWidget *help_menu;
	GtkWidget *help_item;
	GtkWidget *help_view_item;
    GtkWidget *about_separator;
    GtkWidget *about_item;
} jmd_menu_t;

typedef struct {
	gint verbose;
	gint alive;
	gint prefader_count;
	gint postfader_count;
	gint channel_count;
	gint tools_count;
	gint omit_send_pan;
	gint omit_send_gain;
	gint omit_chan_pan;
	gint midi_in_ports;
	gint midi_out_ports;

	gchar* osc_mixer_port;
	gchar* osc_mixer_host;
	gchar client_name[DEFAULT_LABEL_LEN];
	gchar xml_config_file[DEFAULT_LABEL_LEN];
	gchar xml_config_path[256];
	gchar lash_project[DEFAULT_LABEL_LEN];
	gchar lash_server[DEFAULT_LABEL_LEN];
	gchar lash_id[64];

	GtkWidget *clipboard_entry;
	gchar *clipboard_string;
	GtkWidget *clipboard_spinbutton;
	gdouble clipboard_double;
	gint wait;

	jmd_menu_t menu;
	jmd_channel_t *channel;
	jmd_output_t *output;
	jmd_tools_t *tools;

	jmd_midiport_t *in_port;
	jmd_midiport_t *out_port;

	jack_client_t *jack_client;
	lash_client_t *lash_client;

	lo_server_thread st;
	lo_server serv;
	lo_address t;
	lo_timetag ts;

	GtkWidget *window_channels_top;
    GtkWidget *window_channels;
	GtkWidget *box_channels_hbox;

	GtkWidget *window_outputs_top;
    GtkWidget *window_outputs;
	GtkWidget *box_outputs_hbox;



	GtkStyle *spinbutton_style;
	GtkStyle *small_label_style;
	GtkStyle *medium_label_style;
} jmd_interface_t;

static struct option opts_long[] ={
		/* These options set a flag. */
		{"verbose", no_argument,       0, 'v'},
		{"quiet",   no_argument,       0, 'q'},
		/* These options don't set a flag.
		 * We distinguish them by their indices. */
		{"omit-send-pan",	no_argument, 		0, 'a'},
		{"omit-send-gain",	no_argument, 		0, 'g'},
		{"omit-chan-pan",	no_argument, 		0, 'c'},
		{"name",  			required_argument,	0, 'n'},
		{"port",			required_argument,	0, 'p'},
		{"host",    		required_argument,	0, 'h'},
		{"file",    		required_argument,	0, 'f'},
		{"path",    		required_argument,	0, 'd'},
		{"lash-project",	required_argument,	0, 'l'},
		{"lash-server",		required_argument,	0, 's'},
		{"lash-id",			required_argument,	0, 'i'},
		{0, 0, 0, 0}
};

gint running=1;
gint quiet=0;


static void gtk_about_dialog(GtkWidget *widget, gpointer data)
{
	GtkWidget *about_dialog;

	const gchar *authors[8] ={ "","JackMixDesk_gtk by:","\tUlrich Lorenz Schlüter <audio-mobster@gmx.de>",
								"","JackMixDesk by:", "\tNicholas J Humfrey <njh@ecs.soton.ac.uk>",
								"\tUlrich Lorenz Schlüter <audio-mobster@gmx.de>",(const gchar*)NULL };
	static gchar const license[] = "GPLv2";
	gsize cnt = 65536;
	gchar * buf;

	about_dialog=gtk_about_dialog_new();

	gtk_window_set_resizable(GTK_WINDOW(about_dialog),FALSE);
	gtk_window_set_position(GTK_WINDOW(about_dialog),GTK_WIN_POS_CENTER);
	gtk_container_set_border_width(GTK_CONTAINER(about_dialog), 5);
	gtk_window_set_title(GTK_WINDOW(about_dialog), "About JackMixDesk");
	gtk_window_set_keep_above(GTK_WINDOW(about_dialog),TRUE);

	if(g_file_get_contents(INSTALL_DIR "/doc" "/" PACKAGE "-" VERSION "/COPYING", &buf, &cnt,NULL) == TRUE)
		gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about_dialog), buf);
	else
		gtk_about_dialog_set_license(GTK_ABOUT_DIALOG(about_dialog),license);

	free(buf);

	gtk_about_dialog_set_authors(GTK_ABOUT_DIALOG(about_dialog),(const gchar **)&authors);
	gtk_about_dialog_set_name(GTK_ABOUT_DIALOG(about_dialog), "JackMixDesk");
	gtk_about_dialog_set_version(GTK_ABOUT_DIALOG(about_dialog), VERSION);
	gtk_about_dialog_set_copyright(GTK_ABOUT_DIALOG(about_dialog), "Copyright (C) 2007\nUlrich Lorenz Schlüter <audio-mobster@gmx.de>\nNicholas J Humfrey <njh@ecs.soton.ac.uk>");
	gtk_about_dialog_set_comments(GTK_ABOUT_DIALOG(about_dialog),  "JackMixDesk is an Open Source mixer with a configurable number of channels and pre/post sends. It has LASH support an OSC interface and a XML config file. The interface JackMixDesk_gtk has a MIDI mixer and keyboard.");
	gtk_about_dialog_set_website(GTK_ABOUT_DIALOG(about_dialog), "http://sourceforge.net/projects/jackmixdesk/");
	gtk_about_dialog_set_website_label(GTK_ABOUT_DIALOG(about_dialog),"Sourceforge Page");


	g_signal_connect_swapped (G_OBJECT (about_dialog), "response",
			      G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (about_dialog));

	g_signal_connect_swapped (G_OBJECT (about_dialog), "destroy",
			      G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (about_dialog));

	g_signal_connect_swapped (G_OBJECT (about_dialog), "delete_event",
			      G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (about_dialog));

    gtk_widget_show(about_dialog);
}

static void gtk_message_ok_clicked (GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface->wait = 0;
}
static void gtk_message(gchar *message,jmd_interface_t *interface)
{
	GtkWidget *message_dialog;
	GtkWidget *message_button_ok;
	GtkWidget *message_text;

	interface->wait = 1;

	message_text =gtk_label_new(message);

	message_dialog=gtk_dialog_new();
	message_button_ok=gtk_button_new_from_stock(GTK_STOCK_OK);

	gtk_widget_set_size_request(message_dialog,DEFAULT_MESSAGE_WIDTH,DEFAULT_MESSAGE_HEIGHT);
	gtk_window_set_resizable(GTK_WINDOW(message_dialog),FALSE);
	gtk_window_set_position(GTK_WINDOW(message_dialog),GTK_WIN_POS_CENTER);
	gtk_window_set_keep_above(GTK_WINDOW(message_dialog),TRUE);
	gtk_label_set_line_wrap(GTK_LABEL(message_text),TRUE);

	g_signal_connect (G_OBJECT (message_button_ok), "clicked",
			     G_CALLBACK (gtk_message_ok_clicked),(gpointer) interface);

	g_signal_connect_swapped (G_OBJECT (message_button_ok), "clicked",
			     G_CALLBACK (gtk_widget_destroy),G_OBJECT (message_dialog));

	gtk_window_set_title (GTK_WINDOW (message_dialog), "Message!");

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (message_dialog)->vbox),message_text , TRUE, TRUE, 5);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (message_dialog)->action_area),message_button_ok , TRUE, TRUE, 0);

    gtk_widget_show (message_button_ok);
    gtk_widget_show (message_text);

	gtk_dialog_run (GTK_DIALOG(message_dialog));

	while(interface->wait) usleep(100);

}

static void gtk_get_string_entry_change(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;
	interface->clipboard_string = g_malloc0( DEFAULT_LABEL_LEN );

	snprintf(interface->clipboard_string,DEFAULT_LABEL_LEN,"%s",(gchar *)gtk_entry_get_text(GTK_ENTRY(interface->clipboard_entry)));

}

static gchar* gtk_get_string(gchar *message,gchar *entry,jmd_interface_t *interface)
{
	GtkWidget *message_dialog;
	GtkWidget *message_hbox;
	GtkWidget *message_button_ok;
	GtkWidget *message_text;
	GtkWidget *message_entry;
	gchar *s = g_malloc0( DEFAULT_LABEL_LEN );

	message_dialog=gtk_dialog_new();
	message_hbox = gtk_hbox_new(FALSE,0);
	message_text =gtk_label_new(message);
	message_entry = gtk_entry_new();
	message_button_ok=gtk_button_new_from_stock(GTK_STOCK_OK);

	gtk_widget_set_size_request(message_dialog,DEFAULT_MESSAGE_WIDTH,DEFAULT_MESSAGE_HEIGHT);
	gtk_window_set_resizable(GTK_WINDOW(message_dialog),FALSE);
	gtk_window_set_position(GTK_WINDOW(message_dialog),GTK_WIN_POS_CENTER);
	gtk_window_set_keep_above(GTK_WINDOW(message_dialog),TRUE);

	if(entry != NULL) gtk_entry_set_text(GTK_ENTRY(message_entry),entry);
	gtk_entry_set_width_chars(GTK_ENTRY(message_entry),21);
	gtk_entry_set_max_length(GTK_ENTRY(message_entry),jack_port_name_size());

	interface->clipboard_entry = message_entry;
	gtk_entry_set_max_length(GTK_ENTRY(interface->clipboard_entry),jack_port_name_size());


	g_signal_connect (G_OBJECT (message_button_ok), "clicked",
			     G_CALLBACK (gtk_get_string_entry_change),(gpointer) interface);

	g_signal_connect_swapped (G_OBJECT (message_button_ok), "clicked",
			     G_CALLBACK (gtk_widget_destroy),G_OBJECT (message_dialog));

	gtk_window_set_title (GTK_WINDOW (message_dialog), "Enter text!");

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (message_dialog)->vbox),message_hbox , TRUE, TRUE, 5);
	gtk_box_pack_start (GTK_BOX (message_hbox),message_text , TRUE, TRUE, 5);
	gtk_box_pack_start (GTK_BOX (message_hbox),interface->clipboard_entry , TRUE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (message_dialog)->action_area),message_button_ok , TRUE, TRUE, 0);

    gtk_widget_show (message_button_ok);
    gtk_widget_show (message_hbox);
    gtk_widget_show (message_text);
    gtk_widget_show (message_entry);
    gtk_dialog_run (GTK_DIALOG(message_dialog));

    while(interface->clipboard_string == NULL) usleep(100);

    snprintf(s,DEFAULT_LABEL_LEN,"%s",interface->clipboard_string);

   	interface->clipboard_string = NULL;

	if (interface->verbose) {
		g_print("Dialog returns %s\n",s);
	}

    return s;
}

static void gtk_get_double_spinbutton_change(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface->clipboard_double = gtk_spin_button_get_value(GTK_SPIN_BUTTON(interface->clipboard_spinbutton));
}

static int gtk_get_double(gdouble value, gdouble lower, gdouble upper, gdouble step, gchar *message,jmd_interface_t *interface)
{
	GtkWidget *message_dialog;
	GtkWidget *message_hbox;
	GtkWidget *message_button_ok;
	GtkWidget *message_text;
	GtkWidget *message_spinbutton;
	gdouble d;

	message_dialog=gtk_dialog_new();
	message_hbox = gtk_hbox_new(FALSE,0);
	message_text =gtk_label_new(message);
	message_spinbutton = gtk_spin_button_new_with_range(lower,upper,step);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(message_spinbutton), value);

	message_button_ok=gtk_button_new_from_stock(GTK_STOCK_OK);

	gtk_widget_set_size_request(message_dialog,DEFAULT_MESSAGE_WIDTH,DEFAULT_MESSAGE_HEIGHT);
	gtk_window_set_resizable(GTK_WINDOW(message_dialog),FALSE);
	gtk_window_set_position(GTK_WINDOW(message_dialog),GTK_WIN_POS_CENTER);
	gtk_window_set_keep_above(GTK_WINDOW(message_dialog),TRUE);


	interface->clipboard_spinbutton = message_spinbutton;

	g_signal_connect (G_OBJECT (message_button_ok), "clicked",
			     G_CALLBACK (gtk_get_double_spinbutton_change),(gpointer) interface);

	g_signal_connect_swapped (G_OBJECT (message_button_ok), "clicked",
			     G_CALLBACK (gtk_widget_destroy),G_OBJECT (message_dialog));

	gtk_window_set_title (GTK_WINDOW (message_dialog), "Enter number!");

	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (message_dialog)->vbox),message_hbox , TRUE, TRUE, 5);
	gtk_box_pack_start (GTK_BOX (message_hbox),message_text , TRUE, TRUE, 5);
	gtk_box_pack_start (GTK_BOX (message_hbox),interface->clipboard_spinbutton , TRUE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (message_dialog)->action_area),message_button_ok , TRUE, TRUE, 0);

    gtk_widget_show (message_button_ok);
    gtk_widget_show (message_hbox);
    gtk_widget_show (message_text);
    gtk_widget_show (message_spinbutton);
    gtk_dialog_run (GTK_DIALOG(message_dialog));

    while(interface->clipboard_double == -1) usleep(100);
    d = interface->clipboard_double;
    interface->clipboard_double= -1;

	if (interface->verbose) {
    	g_print("Dialog returns %f\n",d);
	}

    return d;
}

static void gtk_fader_gain_out_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gfloat gain=phat_fan_slider_get_value(PHAT_FAN_SLIDER(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->output[i-1].spinbutton_gain)) != gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->output[i-1].spinbutton_gain),gain);
	}
	if ( interface->output[i-1].button_link_state == 1 && phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->output[i].fader_gain))!= gain ){
		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->output[i].fader_gain),gain);
	}
	if ( interface->output[i-2].button_link_state == 1 && phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->output[i-2].fader_gain))!= gain ){
		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->output[i-2].fader_gain),gain);
	}

	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/output/gain_set", "if",i,gain) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
	    g_print ("Gtk out gain fader changed\n");
		g_print ("value=%f\n",phat_fan_slider_get_value(PHAT_FAN_SLIDER(widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
	}
}

static void gtk_spinbutton_gain_out_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gfloat gain=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if ( phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->output[i-1].fader_gain))!= gain ){
		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->output[i-1].fader_gain),gain);
	}

   	if (interface->verbose) {
		g_print ("Gtk out gain entry changed\n");
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
   	}
}

static void gtk_button_mute_out_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

  	if (interface->verbose) {
    	g_print ("Gtk out %i mute button change\n",i);
    	g_print ("%i\n",interface->output[i-1].button_mute_state);
  	}

    if (interface->output[i-1].button_mute_state==0){
    	interface->output[i-1].button_mute_state=1;
    } else {
    	interface->output[i-1].button_mute_state=0;
    }

    if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/output/mute_set", "ii",i,interface->output[i-1].button_mute_state) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
		g_print ("Gtk out %i mute button changed\n",i);
		g_print ("%i\n",interface->output[i-1].button_mute_state);
	}
}

static void gtk_button_link_out_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

  	if (interface->verbose) {
    	g_print ("Gtk out %i link button change\n",i);
    	g_print ("%i\n",interface->output[i-1].button_link_state);
  	}

    if (interface->output[i-1].button_link_state==0){
    	interface->output[i-1].button_link_state=1;
    	if ( phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->output[i-1].fader_gain))!= phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->output[i].fader_gain)) ){
			phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->output[i].fader_gain),phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->output[i-1].fader_gain)));
		}
    } else {
    	interface->output[i-1].button_link_state=0;
    }

    if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/output/link_set", "ii",i,interface->output[i-1].button_link_state) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
		g_print ("Gtk out %i link button changed\n",i);
		g_print ("%i\n",interface->output[i-1].button_link_state);
	}
}

static void gtk_fader_gain_chan_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gfloat gain=phat_fan_slider_get_value(PHAT_FAN_SLIDER(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].spinbutton_gain)) != gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].spinbutton_gain),gain);
	}
	if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[0].knob_gain)) != gain ){
		phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[0].knob_gain),gain);
	}
	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[0].spinbutton_gain)) != gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[0].spinbutton_gain),gain);
	}
	if ( interface->channel[i-1].button_stereo_state == 1 && phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[i].fader_gain))!= gain ){
		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[i].fader_gain),gain);
	}
	if ( interface->channel[i-2].button_stereo_state == 1 && phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[i-2].fader_gain))!= gain ){
		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[i-2].fader_gain),gain);
	}

	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_set", "iif",i,1,gain) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

  	if (interface->verbose) {
	    g_print ("Gtk chan gain fader changed\n");
		g_print ("value=%f\n",phat_fan_slider_get_value(PHAT_FAN_SLIDER(widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
  	}
}

static void gtk_spinbutton_gain_chan_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gfloat gain=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if ( phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[i-1].fader_gain))!= gain ){
		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[i-1].fader_gain),gain);
	}

	if (PHAT_IS_KNOB(interface->channel[i-1].send[0].knob_gain)){
		if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[0].knob_gain)) != gain ){
			phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[0].knob_gain),gain);
		}
	}

		if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[0].spinbutton_gain)) != gain ){
			gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[0].spinbutton_gain),gain);
		}

  	if (interface->verbose) {
	    g_print ("Gtk chan gain entry changed\n");
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
  	}
}

static void gtk_button_mute_chan_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

  	if (interface->verbose) {
    	g_print ("Gtk chan %i mute button change\n",i);
    	g_print ("%i\n",interface->channel[i-1].button_mute_state);
  	}

    if (interface->channel[i-1].button_mute_state==0){
    	interface->channel[i-1].button_mute_state=1;
    } else {
    	interface->channel[i-1].button_mute_state=0;
    }

    if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/mute_set", "ii",i,interface->channel[i-1].button_mute_state) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
		g_print ("Gtk chan %i mute button changed\n",i);
		g_print ("%i\n",interface->channel[i-1].button_mute_state);
	}
}

static void gtk_button_stereo_chan_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gint s;
	gfloat left=-90;
	gfloat right=90;
	gfloat low=-90;
	gfloat high=0;
	jmd_interface_t *interface = (jmd_interface_t*)data;

  	if (interface->verbose) {
    	g_print ("Gtk chan %i stereo button change\n",i);
    	g_print ("%i\n",interface->channel[i-1].button_stereo_state);
  	}

    if (interface->channel[i-1].button_stereo_state==0){
    	interface->channel[i-1].button_stereo_state=1;

		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/stereo_set", "ii",i,1) == -1 ){
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
    	if ( phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[i-1].fader_gain))!= phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[i].fader_gain)) ){
			phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[i].fader_gain),phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[i-1].fader_gain)));
		}
		if (!interface->omit_chan_pan ){
	    	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].spinbutton_pan)) != left ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].spinbutton_pan),left);
	    	}
	    	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i].spinbutton_pan)) != right ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i].spinbutton_pan),right);
	    	}
		}

	 	for ( s=0;s<(interface->prefader_count+interface->postfader_count);s++){
			if (!interface->omit_send_pan){
				if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_pan)) != left ){
					phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_pan),left);
				}
				if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_pan)) != left ){
					gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_pan),left);
				}
				if (phat_knob_get_value(PHAT_KNOB (interface->channel[i].send[s].knob_pan)) != right ){
					phat_knob_set_value(PHAT_KNOB (interface->channel[i].send[s].knob_pan),right);
				}
				if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i].send[s].spinbutton_pan)) != right ){
					gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i].send[s].spinbutton_pan),right);
				}
			}
			if (!interface->omit_send_gain){
				if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_l)) != high ){
					phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_l),high);
				}
				if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_l)) != high ){
					gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_l),high);
				}
				if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_r)) != low ){
					phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_r),low);
				}
				if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_r)) != low ){
					gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_r),low);
				}
				if (phat_knob_get_value(PHAT_KNOB (interface->channel[i].send[s].knob_gain_l)) != low ){
					phat_knob_set_value(PHAT_KNOB (interface->channel[i].send[s].knob_gain_l),low);
				}
				if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i].send[s].spinbutton_gain_l)) != low ){
					gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i].send[s].spinbutton_gain_l),low);
				}
				if (phat_knob_get_value(PHAT_KNOB (interface->channel[i].send[s].knob_gain_r)) != high ){
					phat_knob_set_value(PHAT_KNOB (interface->channel[i].send[s].knob_gain_r),high);
				}
				if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i].send[s].spinbutton_gain_r)) != high ){
					gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i].send[s].spinbutton_gain_r),high);
				}
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",i,s+1,high) == -1 ){
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",i,s+1,low) == -1 ){
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",i+1,s+1,low) == -1 ){
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",i+1,s+1,high) == -1 ){
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
		}
    } else {
    	interface->channel[i-1].button_stereo_state=0;
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/stereo_set", "ii",i,0) == -1 ){
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	}

	if (interface->verbose) {
		g_print ("Gtk chan %i stereo button changed\n",i);
		g_print ("%i\n",interface->channel[i-1].button_stereo_state);
	}
}

static void gtk_knob_pan_chan_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gint s;
	gfloat pan=phat_knob_get_value(PHAT_KNOB(widget));
	gfloat gain_l,gain_r;
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if (interface->verbose) {
	    g_print ("Gtk chan pan fader change\n");
		g_print ("value=%f\n",phat_knob_get_value(PHAT_KNOB(widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
	}

	if ( pan < 0 ){
		gain_l = 0;
		gain_r = pan;
	} else if ( pan > 0 ){
		gain_l = pan * (-1);
		gain_r = 0;
	} else {
		gain_l = 0;
		gain_r = 0;
	}

	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].spinbutton_pan)) != pan ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].spinbutton_pan),pan);
	}

	for ( s=0;s<(interface->prefader_count+interface->postfader_count);s++){
		if (!interface->omit_send_pan){
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_pan)) != pan ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_pan),pan);
			}
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_pan)) != pan ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_pan),pan);
			}
		}
		if (!interface->omit_send_gain){
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_l)) != gain_l ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_l),gain_l);
			}
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_l)) != gain_l ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_l),gain_l);
			}
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_r)) != gain_r ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_gain_r),gain_r);
			}
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_r)) != gain_r ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_gain_r),gain_r);
			}
		}

		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",i,s+1,gain_l) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",i,s+1,gain_r) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	}


	if (interface->verbose) {
	    g_print ("Gtk chan pan fader changed\n");
		g_print ("value=%f\n",phat_knob_get_value(PHAT_KNOB(widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
	}
}

static void gtk_spinbutton_pan_chan_change(GtkWidget *widget, gpointer data)
{
	gint i=atoi(gtk_widget_get_name(widget));
	gfloat pan=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;
	gint s;

	if (interface->verbose) {
	    g_print ("Gtk chan pan entry changed\n");
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
	}

	if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].knob_pan)) != pan ){
		phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].knob_pan),pan);
	}

	for (s=0;s<interface->prefader_count+interface->postfader_count;s++){
		if (PHAT_IS_KNOB(interface->channel[i-1].send[s].knob_pan)){
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_pan)) != pan ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[i-1].send[s].knob_pan),pan);
			}
		}
		if (GTK_IS_SPIN_BUTTON(interface->channel[i-1].send[s].spinbutton_pan)){
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_pan)) != pan ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[i-1].send[s].spinbutton_pan),pan);
			}
		}
	}

	if (interface->verbose) {
	    g_print ("Gtk chan pan entry changed\n");
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("%i\n",atoi(gtk_widget_get_name(widget)));
	}
}

static void gtk_knob_gain_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain=phat_knob_get_value(PHAT_KNOB(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if(s == 1) phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[c-1].fader_gain),gain);

	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain)) != gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain),gain);
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_set", "iif",c,s,gain) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	}
	if ( interface->channel[c-1].button_stereo_state == 1 && gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c].send[s-1].spinbutton_gain))!= gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c].send[s-1].spinbutton_gain),gain);
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_set", "iif",c+1,s,gain) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	}
	if ( interface->channel[c-2].button_stereo_state == 1 && gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-2].send[s-1].spinbutton_gain))!= gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-2].send[s-1].spinbutton_gain),gain);
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_set", "iif",c-1,s,gain) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	}

	if (interface->verbose) {
	    g_print ("Gtk send gain fader changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",phat_knob_get_value(PHAT_KNOB(widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_spinbutton_gain_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain)) != gain ){
		phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain),gain);
	}
	if(s == 1){
		if ( phat_fan_slider_get_value(PHAT_FAN_SLIDER(interface->channel[c-1].fader_gain))!= gain ){
			phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[c-1].fader_gain),gain);
		}
	}
	if (interface->verbose) {
	    g_print ("Gtk send gain entry changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_knob_pan_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain_l,gain_r;
	gfloat pan=phat_knob_get_value(PHAT_KNOB (widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	if ( pan < 0 ){
		gain_l = 0;
		gain_r = pan;
	} else if ( pan > 0 ){
		gain_l = pan * (-1);
		gain_r = 0;
	} else {
		gain_l = 0;
		gain_r = 0;
	}

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if (!interface->omit_send_pan){
		if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_pan)) != pan ){
			gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_pan),pan);
		}
	}

	if (!interface->omit_send_gain){
		if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l)) != gain_l ){
			phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l),gain_l);
		}
		if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l)) != gain_l ){
			gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l),gain_l);
		}
		if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r)) != gain_r ){
			phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r),gain_r);
		}
		if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r)) != gain_r ){
			gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r),gain_r);
		}
	}

	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",c,s,gain_l) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}
	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",c,s,gain_r) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
	    g_print ("Gtk send pan fader changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",phat_knob_get_value(PHAT_KNOB(widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_spinbutton_pan_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat pan=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if(s-1 == 0){
		if (GTK_IS_SPIN_BUTTON(interface->channel[c-1].spinbutton_pan)){
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].spinbutton_pan)) != pan ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].spinbutton_pan),pan);
			}
		}
		if (PHAT_IS_KNOB(interface->channel[c-1].knob_pan)){
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].knob_pan)) != pan ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].knob_pan),pan);
			}
		}
	}

	if (PHAT_IS_KNOB(interface->channel[c-1].send[s-1].knob_pan)){
		if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_pan)) != pan ){
			phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_pan),pan);
		}
	}

	if (PHAT_IS_KNOB(interface->channel[c-1].send[s-1].knob_gain_l) && PHAT_IS_KNOB(interface->channel[c-1].send[s-1].knob_gain_r)){
		if (pan <= 0){
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l)) != 0 ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l),0);
			}
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r)) != pan ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r),pan);
			}
		}
		if (pan >= 0){
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l)) != pan  * (-1) ){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l),pan  * (-1));
			}
			if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r)) !=  0){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r),0);
			}
		}
	}
	if (GTK_IS_SPIN_BUTTON(interface->channel[c-1].send[s-1].spinbutton_gain_l) && GTK_IS_SPIN_BUTTON(interface->channel[c-1].send[s-1].spinbutton_gain_r)){
		if (pan <= 0){
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l)) != 0 ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l),0);
			}
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r)) != pan ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r),pan);
			}
		} else if (pan >= 0){
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l)) != pan  * (-1) ){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l),pan  * (-1));
			}
			if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r)) != 0){
				gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r),0);
			}
		}
	}

	if (pan <= 0){
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",c,s,0.0) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",c,s,pan) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	} else {
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",c,s,pan * (-1)) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",c,s,0.0) == -1 ){
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
	}

	if (interface->verbose) {
	    g_print ("Gtk send pan entry changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_knob_gain_l_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain=phat_knob_get_value(PHAT_KNOB(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l)) != gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_l),gain);
	}

	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_l_set", "iif",c,s,gain) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
	    g_print ("Gtk send gain left fader changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",phat_knob_get_value(PHAT_KNOB(widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_spinbutton_gain_l_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l)) != gain ){
		phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_l),gain);
	}

	if (interface->verbose) {
	    g_print ("Gtk send gain left entry changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_knob_gain_r_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain=phat_knob_get_value(PHAT_KNOB(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if (gtk_spin_button_get_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r)) != gain ){
		gtk_spin_button_set_value(GTK_SPIN_BUTTON (interface->channel[c-1].send[s-1].spinbutton_gain_r),gain);
	}

	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE,"/mixer/channel/send/gain_r_set", "iif",c,s,gain) == -1 ){
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	if (interface->verbose) {
	    g_print ("Gtk send gain right fader changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",phat_knob_get_value(PHAT_KNOB(widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_spinbutton_gain_r_send_change(GtkWidget *widget, gpointer data)
{
	gint i,h,c,s;
	gchar chan[10];
	gchar send[10];
	gchar name[10];
	gfloat gain=gtk_spin_button_get_value(GTK_SPIN_BUTTON(widget));
	jmd_interface_t *interface = (jmd_interface_t*)data;

	sprintf(name,"%s",gtk_widget_get_name(widget));

	i=0;
	h=0;
	while (name[i] != ','){
		chan[i] = name[i];
		i++;
	}
	if (name[i] == ','){
		chan[i] = '\0';
		i++;
	}
	while (name[i] != '\0'){
		send[h] = name[i];
		i++;
		h++;
	}
	if (name[i] == '\0'){
		send[h] = '\0';
	}

	c=atoi(chan);
	s=atoi(send);

	if (phat_knob_get_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r)) != gain ){
		phat_knob_set_value(PHAT_KNOB (interface->channel[c-1].send[s-1].knob_gain_r),gain);
	}

	if (interface->verbose) {
	    g_print ("Gtk send gain right entry changed\n");
		g_print ("name=%s\n",gtk_widget_get_name(widget));
		g_print ("value=%f\n",gtk_spin_button_get_value(GTK_SPIN_BUTTON (widget)));
		g_print ("channel=%i send=%i\n",c,s);
	}
}

static void gtk_destroy(GtkWidget *widget, gpointer data)
{
	running=0;
    gtk_main_quit ();
	g_print ("destroy event\n");
}

static void gtk_delete(GtkWidget *widget, gpointer data)
{
	running=0;
    gtk_main_quit ();
	g_print ("delete event\n");
}

static void gtk_toggle_widget_view(GtkWidget *widget, gpointer data)
{
	gtk_widget_hide(widget);
}

jack_port_t* jack_create_in_port(const gchar* name, jmd_interface_t *interface)
{
	gint port_name_size = jack_port_name_size();
	gchar *port_name = g_malloc0( port_name_size );
	jack_port_t *port;

	snprintf( port_name, port_name_size, "%s", name );
	port = jack_port_register(interface->jack_client, port_name, JACK_DEFAULT_MIDI_TYPE, JackPortIsInput, 0);
	if (!port) {
		fprintf(stderr, "Cannot register input port '%s'.\n", port_name);
		exit(1);
	}

	g_free(port_name);

	return port;
}

jack_port_t* jack_create_out_port(const gchar* name, jmd_interface_t *interface)
{
	gint port_name_size = jack_port_name_size();
	gchar *port_name = g_malloc0( port_name_size );
	jack_port_t *port;

	snprintf( port_name, port_name_size,"%s",name);
	port = jack_port_register(interface->jack_client, port_name, JACK_DEFAULT_MIDI_TYPE, JackPortIsOutput, 0);
	if (!port) {
		fprintf(stderr, "Cannot register output port '%s'.\n", port_name);
		exit(1);
	}

	g_free(port_name);

	return port;
}

static jmd_interface_t* interface_init()
{
	jmd_interface_t *interface;
	interface = g_malloc(sizeof(jmd_interface_t));
	interface->verbose = 0;
	interface->alive = 0;
	interface->channel_count=0;
	interface->prefader_count=0;
	interface->postfader_count=0;
	interface->tools_count=0;
	interface->midi_in_ports=0;
	interface->midi_out_ports=0;
	interface->omit_send_pan=0;
	interface->omit_send_gain=0;
	interface->osc_mixer_port=DEFAULT_MIXER_PORT;
	interface->osc_mixer_host=DEFAULT_MIXER_HOST;
	sprintf(interface->xml_config_file,"%s",DEFAULT_CONFIG_FILE);
	sprintf(interface->xml_config_path,"%s",DEFAULT_CONFIG_PATH);
	sprintf(interface->client_name, "%s", DEFAULT_CLIENT_NAME );

	interface->clipboard_string=NULL;
	interface->clipboard_double = -1;
	interface->wait = 0;
	interface->st =NULL;

	interface->tools=NULL;

	interface->in_port = g_malloc0(sizeof(jmd_midiport_t *));
	interface->out_port = g_malloc0(sizeof(jmd_midiport_t *));

	return(interface);
}

static gint interface_usage()
{
	g_print("Copyright (C) 2007 Ulrich Lorenz Schlueter <audio-mobster@gmx.de>\n\n");
	g_print("This program comes with ABSOLUTELY NO WARRANTY.  You are licensed to use it\n");
	g_print("under the terms of the GNU General Public License, version 2 or later.  See\n");
	g_print("the COPYING file that came with this software for details.\n\n");
	g_print("JackMixDesk_gtk\n\n");
	g_print("   -p <port>     Set the UDP port number for jackmixdesk.\n");
	g_print("   -h <host>     Set the UDP hostname for jackmixdesk (default localhost).\n");
	g_print("   -n <name>     Name for this OSC client (default jackmixdesk_gtk).\n");
	g_print("   -d <path>     Path for config file (default $HOME).\n");
	g_print("   -f <file>     Name for config file (default .mixdeskgtkrc).\n");
	g_print("   -s            Omit send pan knobs use send gain left and right instead.\n");
	g_print("   -g            Omit send send gain left and right knobs use pan instead.\n");
	g_print("   -c            Omit main channel pan use....\n");
	g_print("   -v            Enable verbose mode.\n");
	g_print("   -q            Enable quiet mode.\n");
	g_print("\n");
	exit(1);
}

static gint interface_welcome(jmd_interface_t *interface)
{
    if (!quiet) {
		gchar *url = lo_server_thread_get_url(interface->st);
		g_print( "%s\nOSC server URL: %s\n", interface->client_name, url );
		g_free(url);
	}

	/* Dislay welcoming message */
	if (interface->verbose) {
		g_print("\nStarting %s with %i channels %i prefaders %i postfaders.\n",
			interface->client_name, interface->channel_count,interface->prefader_count,interface->postfader_count);
		g_print("JackMixDesk OSC Port: %s\n",interface->osc_mixer_port);
	}
	return 0;
}

static void interface_signal_handler(int signum)
{
	if (!quiet) {
		switch(signum) {
			case SIGTERM:	fprintf(stderr, "Got termination signal.\n"); break;
			case SIGINT:	fprintf(stderr, "Got interupt signal.\n"); break;
		}
	}
	running=0;
	gtk_main_quit();
	g_print("Quitting JackMixDesk_gtk.\n");
}

static gint interface_mixer_alive(jmd_interface_t *interface){

	gint timeout = 0;
	while(!interface->alive){
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/ping", "") == -1) {
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		sleep(1);
		timeout ++;
		if(timeout == TIMEOUT) {
			g_print ("Could not connect to mixer.\n");
			g_print ("Midi utilities only.\n");
			g_print ("Run jackmixdesk first!\n");
			gtk_message("Could not connect to mixer. Midi utilities only. Run jackmixdesk first!",interface);
			return 1;
		}
	}
	return 0;
}

static gint interface_init_mixer(jmd_interface_t *interface){

	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel_count_get", "") == -1) {
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}
	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/prefader_count_get", "") == -1) {
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}
	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/postfader_count_get", "") == -1) {
		g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
	}

	return 0;
}

static gint interface_init_channels_struct(jmd_interface_t *interface)
{
	gint i;

	interface->channel = g_malloc(
		sizeof(jmd_channel_t) * interface->channel_count);

	for (i=0;i<interface->channel_count;i++){
		interface->channel[i].send = g_malloc(
			sizeof(jmd_send_t) * (interface->prefader_count+interface->postfader_count));
	}
	return 0;
}

static gint interface_get_channels_data(jmd_interface_t *interface)
{
	gint i,h;

	for (i=0;i<interface->channel_count;i++){

	    if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/label_get", "i",i+1) == -1) {
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if(i < interface->channel_count-1){
	    	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/stereo_get", "i",i+1) == -1) {
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
		}
	    if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/mute_get", "i",i+1) == -1) {
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}

		for (h=0;h<interface->prefader_count+interface->postfader_count;h++){

			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/send/gain_get", "ii",i+1,h+1) == -1) {
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/send/pan_get", "ii",i+1,h+1) == -1) {
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/send/gain_l_get", "ii",i+1,h+1) == -1) {
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
			if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/channel/send/gain_r_get", "ii",i+1,h+1) == -1) {
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
		}
	}
	return 0;
}

/*static gint interface_init_midichannel_struct(jmd_midimix_channel_t *channel)
{
	gint c=16;

	if(channel == NULL) channel = g_malloc0(sizeof(jmd_midimix_channel_t) * c);

    return 0;
}

static gint interface_init_keyboard_struct(jmd_keyboard_t **keyboard)
{
	gint i=0;

	if(keyboard == NULL) keyboard = g_malloc0(sizeof(jmd_keyboard_t *));

    while(keyboard[i] != NULL){
    	i++;
    }*/
    /* if NULL realloc keyboard pointer array one bigger
     * malloc new keyboard with new size
     * set last pointer to NULL */
/*    if(keyboard[i] == NULL){
		keyboard[i] = g_malloc0(sizeof(jmd_keyboard_t));
		keyboard=g_realloc(keyboard,sizeof(jmd_keyboard_t *) * (i+2));
    	keyboard[i+1] = NULL;
    }
    return 0;
}

static gint interface_init_phatpad_struct(jmd_phatpad_t **phatpad)
{
	gint i=0;

	if(phatpad == NULL) phatpad = g_malloc0(sizeof(jmd_phatpad_t *));

    while(phatpad[i] != NULL){
    	i++;
    }*/
    /* if NULL realloc phatpad pointer array one bigger
     * malloc new phatpad with new size
     * set last pointer to NULL */
    /*if(phatpad[i] == NULL){
		phatpad[i] = g_malloc0(sizeof(jmd_phatpad_t));
		phatpad=g_realloc(phatpad,sizeof(jmd_phatpad_t *) * (i+2));
    	phatpad[i+1] = NULL;
    }
    return 0;
}*/

static gint interface_init_midimixers_struct(jmd_interface_t *interface)
{
    return 0;
}

static gint interface_init_keyboards_struct(jmd_interface_t *interface)
{
    return 0;
}

static gint interface_init_phatpads_struct(jmd_interface_t *interface)
{
    return 0;
}

static gint interface_init_tools_struct(jmd_interface_t *interface)
{
	interface->tools_count += 1;

	interface->tools=g_realloc(interface->tools,sizeof(jmd_tools_t) * (interface->tools_count));

    return 0;
}

static gint interface_add_midimix(jmd_interface_t *interface)
{
    gtk_get_double(16,1,16,1,"Select number of MIDI channels:",interface);

    return 0;
}

static gint interface_add_keyboard(jmd_interface_t *interface)
{
    gtk_get_double(88,1,128,1,"Select number of keys:",interface);

    return 0;
}

static gint interface_del_midi_in_port(jmd_interface_t *interface)
{
    /* realloc pointer array one smaller */
    if(interface->midi_in_ports >= 1){
		jack_port_unregister(interface->jack_client, interface->in_port[interface->midi_in_ports-1].port);
		interface->in_port=g_realloc(interface->in_port,sizeof(jmd_midiport_t) * (interface->midi_in_ports-1));
		interface->midi_in_ports -= 1;
    } else {
    	gtk_message("No port to delete!",interface);
    }
    return 0;
}

static gint interface_del_midi_out_port(jmd_interface_t *interface)
{
    /* realloc pointer array one smaller */
    if(interface->midi_out_ports >= 1){
		jack_port_unregister(interface->jack_client, interface->out_port[interface->midi_out_ports-1].port);
		interface->out_port=g_realloc(interface->out_port,sizeof(jmd_midiport_t) * (interface->midi_out_ports-1));
		interface->midi_out_ports -= 1;
    } else {
    	gtk_message("No port to delete!",interface);
    }
    return 0;
}

static void gtk_menu_add_midi_in(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;
	gint port_name_size = jack_port_name_size();
	gchar *port_name = g_malloc0( port_name_size );

	interface->in_port=g_realloc(interface->in_port,
						sizeof(jmd_midiport_t) * (interface->midi_in_ports+1));

	snprintf(port_name,port_name_size,"%s",gtk_get_string("Enter in port name:","in port",interface));

	interface->in_port[interface->midi_in_ports].port = jack_create_in_port(port_name, interface);
	interface->midi_in_ports += 1;

	g_free(port_name);
}

static void gtk_menu_add_midi_out(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;
	gint port_name_size = jack_port_name_size();
	gchar *port_name = g_malloc0( port_name_size );

	interface->out_port=g_realloc(interface->out_port,
						sizeof(jmd_midiport_t) * (interface->midi_out_ports+1));

	snprintf(port_name,port_name_size,"%s",gtk_get_string("Enter out port name","out port",interface));

	interface->out_port[interface->midi_out_ports].port = jack_create_out_port(port_name, interface);
	interface->midi_out_ports += 1;

	g_free(port_name);
}

static void gtk_menu_del_midi_in(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface_del_midi_in_port(interface);
}

static void gtk_menu_del_midi_out(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface_del_midi_out_port(interface);
}

static void gtk_menu_keyboard(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface_init_keyboards_struct(interface);
	interface_init_tools_struct(interface);

	interface_add_keyboard(interface);
}

static void gtk_menu_phatpad(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface_init_phatpads_struct(interface);
	interface_init_tools_struct(interface);
}

static void gtk_menu_midimix(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	interface_init_midimixers_struct(interface);
	interface_init_tools_struct(interface);

	interface_add_midimix(interface);
}

static void gtk_menu_help(GtkWidget *widget, gpointer data)
{
	GtkWidget *window_top;
	GtkWidget *notebook;
	GtkWidget *vbox_1;
	GtkWidget *label_1;
	GtkWidget *window_1;
	GtkTextIter iter;
	GtkTextBuffer *readme_buffer;
	GtkWidget *readme_view;
	GtkWidget *vbox_2;
	GtkWidget *label_2;
	GtkWidget *window_2;
	GdkPixbuf *pixbuf=NULL;
	GtkWidget *image;
	gsize cnt = 65536;
	gchar *buf=NULL;

	pixbuf = gdk_pixbuf_new_from_file_at_size(INSTALL_DIR "/doc" "/" PACKAGE "-"
													VERSION "/jackmixdesk.png",1280,-1,NULL);
	if(g_file_get_contents(INSTALL_DIR "/doc" "/" PACKAGE "-" VERSION "/README", &buf, &cnt,NULL) == TRUE);
	else buf = "README file not found!";

	window_top = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	notebook = gtk_notebook_new();
	vbox_1 = gtk_vbox_new(FALSE,0);
	label_1 = gtk_label_new("README");
	window_1 = gtk_scrolled_window_new(NULL,NULL);

	readme_buffer = gtk_text_buffer_new (NULL);
	vbox_2 = gtk_vbox_new(FALSE,0);
	label_2 = gtk_label_new("Signalpath");
	window_2 = gtk_scrolled_window_new(NULL,NULL);

    image = gtk_image_new_from_pixbuf (pixbuf);

	gtk_window_set_default_size(GTK_WINDOW(window_top),560,580);

	gtk_text_buffer_get_start_iter (readme_buffer, &iter);
	gtk_text_buffer_insert (readme_buffer, &iter,buf, -1);
	readme_view = gtk_text_view_new_with_buffer (readme_buffer);

	g_signal_connect_swapped (G_OBJECT (window_top), "delete_event",
			      G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (window_top));

	g_signal_connect_swapped (G_OBJECT (window_top), "destroy",
			      G_CALLBACK (gtk_widget_destroy),
                              G_OBJECT (window_top));

	gtk_window_set_title (GTK_WINDOW (window_top), "JackMixDesk Help");
	gtk_window_set_position(GTK_WINDOW(window_top),GTK_WIN_POS_CENTER);

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (window_1),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (window_2),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    gtk_container_add (GTK_CONTAINER (window_top),notebook);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox_1,label_1);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),vbox_2,label_2);
	gtk_container_add (GTK_CONTAINER (vbox_1),window_1);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(window_1), readme_view);

	gtk_container_add (GTK_CONTAINER (vbox_2),window_2);

	/*gtk_container_add (GTK_CONTAINER (window_2), image);*/
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(window_2), image);

    gtk_widget_show_all(window_top);

    g_free(buf);
}

static void gtk_menu_update_window_item(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if(!GTK_WIDGET_VISIBLE(interface->window_outputs_top)) {
		if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(interface->menu.output_show_item))) {
			gtk_check_menu_item_set_inconsistent(GTK_CHECK_MENU_ITEM(interface->menu.output_show_item),TRUE);
		} else {
			gtk_check_menu_item_set_inconsistent(GTK_CHECK_MENU_ITEM(interface->menu.output_show_item),FALSE);
		}
	}
	if(!GTK_WIDGET_VISIBLE(interface->window_channels_top)) {
		if(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(interface->menu.channel_show_item))) {
			gtk_check_menu_item_set_inconsistent(GTK_CHECK_MENU_ITEM(interface->menu.channel_show_item),TRUE);
		} else {
			gtk_check_menu_item_set_inconsistent(GTK_CHECK_MENU_ITEM(interface->menu.channel_show_item),FALSE);
		}
	}
}

static gint interface_init_outputs_struct(jmd_interface_t *interface)
{
	interface->output = g_malloc(
		sizeof(jmd_output_t) * (interface->prefader_count+interface->postfader_count));

	return 0;
}

static gint interface_get_outputs_data(jmd_interface_t *interface)
{
	gint i;

	for (i=0;i<interface->prefader_count+interface->postfader_count;i++){
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/output/label_get", "i",i+1) == -1) {
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/output/gain_get", "i",i+1) == -1) {
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/output/mute_get", "i",i+1) == -1) {
			g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
		}
		if(i < interface->prefader_count + interface->postfader_count - 1){
	    	if ( lo_send_from(interface->t,interface->serv,LO_TT_IMMEDIATE, "/mixer/output/link_get", "i",i+1) == -1) {
				g_print("OSC error %d: %s\n", lo_address_errno(interface->t), lo_address_errstr(interface->t));
			}
		}
	}
	return 0;
}

static gint interface_init_style(jmd_interface_t *interface){

    /* Style for spin buttons */
    interface->spinbutton_style = gtk_style_new();
    interface->small_label_style = gtk_style_new();
    interface->medium_label_style = gtk_style_new();

    /* set fonts */
    pango_font_description_set_absolute_size(interface->spinbutton_style->font_desc,SPINBUTTON_FONT_SIZE);
    pango_font_description_set_absolute_size(interface->small_label_style->font_desc,SMALL_LABEL_FONT_SIZE);
    pango_font_description_set_absolute_size(interface->medium_label_style->font_desc,MEDIUM_LABEL_FONT_SIZE);
	pango_font_description_set_family(interface->spinbutton_style->font_desc,"bold");
	pango_font_description_set_family(interface->small_label_style->font_desc,"bold");
	pango_font_description_set_family(interface->medium_label_style->font_desc,"bold");

	return 0;
}

static void gtk_toggle_window_channels(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if(GTK_WIDGET_VISIBLE(interface->window_channels_top)){
		gtk_widget_hide(interface->window_channels_top);
	} else {
		gtk_widget_show(interface->window_channels_top);
	}
}

static void gtk_toggle_window_outputs(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if(GTK_WIDGET_VISIBLE(interface->window_outputs_top)){
		gtk_widget_hide(interface->window_outputs_top);
	} else {
		gtk_widget_show(interface->window_outputs_top);
	}
}

static void gtk_menu_menu_above(GtkWidget *widget, gpointer data)
{
	jmd_interface_t *interface = (jmd_interface_t*)data;

	if(interface->menu.keep_above == 0){
    	gtk_window_set_keep_above(GTK_WINDOW(interface->menu.window),TRUE);
    	interface->menu.keep_above = 1;
	} else {
    	gtk_window_set_keep_above(GTK_WINDOW(interface->menu.window),FALSE);
    	interface->menu.keep_above = 0;
	}
}

static gint interface_init_gtk_channels(jmd_interface_t *interface)
{
	gchar window_title[64];
	gchar lc_name[5];
	gchar *utf_name=NULL;
	gint c,s;
	gfloat pan;
	gchar* chan_label;
	gchar* out_label;

    interface->window_channels_top = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    interface->box_channels_hbox = gtk_hbox_new (FALSE, 0);

    gtk_widget_show (interface->box_channels_hbox);

	interface->window_channels = gtk_scrolled_window_new(NULL, NULL);

	sprintf( window_title,"JackMixDesk Channels 1 - %i", interface->channel_count);
	gtk_window_set_title (GTK_WINDOW (interface->window_channels_top), window_title);

    gtk_container_add (GTK_CONTAINER (interface->window_channels_top),interface->window_channels );

	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (interface->window_channels), interface->box_channels_hbox);

	g_signal_connect_swapped (G_OBJECT (interface->window_channels_top), "delete_event",
			      G_CALLBACK (gtk_toggle_widget_view),
                              G_OBJECT (interface->window_channels_top));

	g_signal_connect_swapped (G_OBJECT (interface->window_channels_top), "destroy",
			      G_CALLBACK (gtk_toggle_widget_view),
                              G_OBJECT (interface->window_channels_top));

    /* Sets the border width of the window. */
    gtk_container_set_border_width (GTK_CONTAINER (interface->window_channels_top), 0);
    gtk_container_set_border_width (GTK_CONTAINER (interface->window_channels), 0);


	gtk_widget_set_size_request(GTK_WIDGET(interface->window_channels_top),
									INPUT_WIDTH + SEPARATOR_WIDTH + CHANNEL_OFFSET, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (interface->window_channels),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);

    if(interface->channel_count < 7) {
		gtk_window_set_default_size (GTK_WINDOW(interface->window_channels_top),
										interface->channel_count * (INPUT_WIDTH + SEPARATOR_WIDTH) + CHANNEL_OFFSET, 580);
	} else {
		gtk_window_set_default_size (GTK_WINDOW(interface->window_channels_top),
										8 * (INPUT_WIDTH + SEPARATOR_WIDTH) + CHANNEL_OFFSET, 580);
	}

	/* create the widgets for the channel window */
    for(c=0;c<interface->channel_count;c++){

		if (interface->channel[c].send[0].gain_l == 0 && interface->channel[c].send[0].gain_r < 0){
			pan = interface->channel[c].send[0].gain_r;
		} else if (interface->channel[c].send[0].gain_r == 0 && interface->channel[c].send[0].gain_l < 0){
			pan = interface->channel[c].send[0].gain_l * (-1);
		} else {
			pan = 0;
		}

		chan_label = interface->channel[c].label;
		out_label = interface->output[0].label;

		interface->channel[c].separator = gtk_vseparator_new ();
		gtk_widget_set_size_request(interface->channel[c].separator,SEPARATOR_WIDTH,-1);
    	interface->channel[c].box_vbox = gtk_vbox_new (FALSE, 0);
		gtk_widget_set_size_request(interface->channel[c].box_vbox,INPUT_WIDTH,-1);
    	interface->channel[c].box_hbox_button_stereo = gtk_hbox_new (FALSE, 0);
    	gtk_box_set_spacing(GTK_BOX(interface->channel[c].box_vbox),BOX_VSPACING);
		gtk_widget_set_size_request(interface->channel[c].box_hbox_button_stereo,-1,BUTTON_SIZE);
		gtk_box_set_spacing(GTK_BOX(interface->channel[c].box_hbox_button_stereo),BOX_HSPACING);

    	interface->channel[c].box_hbox_button_mute = gtk_hbox_new (FALSE, 0);
    	gtk_box_set_spacing(GTK_BOX(interface->channel[c].box_hbox_button_mute),BOX_HSPACING);
    	interface->channel[c].box_hbox_entry_gain = gtk_hbox_new (FALSE, 0);
    	interface->channel[c].label_gain = gtk_label_new("Gain");
    	interface->channel[c].fader_gain = phat_vfan_slider_new_with_range(interface->channel[c].send[0].gain,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
		/* omit for last channel */
		if( c < interface->channel_count -1){
     		interface->channel[c].spacer_stereo = gtk_hbox_new (FALSE, 0);
    		interface->channel[c].label_stereo = gtk_label_new("Stereo");
     		interface->channel[c].button_stereo = gtk_toggle_button_new();
     		gtk_widget_set_size_request(interface->channel[c].button_stereo,BUTTON_SIZE,BUTTON_SIZE);
     		gtk_widget_set_style(interface->channel[c].label_stereo,interface->small_label_style);
		}
     	interface->channel[c].spacer_mute = gtk_hbox_new (FALSE, 0);
     	interface->channel[c].button_mute = gtk_toggle_button_new();
		gtk_widget_set_size_request(GTK_WIDGET(interface->channel[c].button_mute),BUTTON_SIZE,BUTTON_SIZE);

     	interface->channel[c].label_mute = gtk_label_new("Mute");
     	gtk_widget_set_style(interface->channel[c].label_mute,interface->small_label_style);
		interface->channel[c].spinbutton_gain = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
		gtk_widget_set_style(interface->channel[c].spinbutton_gain,interface->spinbutton_style);
    	interface->channel[c].label_channel_top = gtk_label_new(chan_label);

		interface->channel[c].separator_top = gtk_hseparator_new ();
		interface->channel[c].separator_buttons = gtk_hseparator_new ();
		interface->channel[c].separator_bottom = gtk_hseparator_new ();
    	interface->channel[c].label_channel_bottom = gtk_label_new(chan_label);
    	interface->channel[c].label_channel_out = gtk_label_new(out_label);
    	gtk_widget_set_style(interface->channel[c].label_channel_out,interface->medium_label_style);
    	gtk_widget_set_style(interface->channel[c].label_gain,interface->small_label_style);
    	gtk_widget_set_style(interface->channel[c].button_mute,interface->small_label_style);
		gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].spinbutton_gain),DEFAULT_ENTRY_LEN);
		gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].spinbutton_gain),DEFAULT_ENTRY_DIGITS);
		gtk_widget_set_style(interface->channel[c].spinbutton_gain,interface->spinbutton_style);

		sprintf(lc_name,"%i",c+1);
		utf_name = stringprep_locale_to_utf8(lc_name);
		gtk_widget_set_name (interface->channel[c].label_channel_top,utf_name);
		gtk_widget_set_name (interface->channel[c].fader_gain,utf_name);
		gtk_widget_set_name (interface->channel[c].spinbutton_gain,utf_name);
		/* omit for last channel */
		if( c < interface->channel_count -1){
    		gtk_widget_set_name (interface->channel[c].button_stereo,utf_name);
		}
		gtk_widget_set_name (interface->channel[c].button_mute,utf_name);


		g_signal_connect (G_OBJECT (interface->channel[c].fader_gain), "value_changed",
				G_CALLBACK (gtk_fader_gain_chan_change),(gpointer) interface );

		g_signal_connect (G_OBJECT (interface->channel[c].spinbutton_gain), "value_changed",
				G_CALLBACK (gtk_spinbutton_gain_chan_change),(gpointer) interface );

		/* omit for last channel */
		if( c < interface->channel_count -1){
    			g_signal_connect (G_OBJECT (interface->channel[c].button_stereo), "clicked",
					G_CALLBACK (gtk_button_stereo_chan_change),(gpointer) interface );
		}

		g_signal_connect (G_OBJECT (interface->channel[c].button_mute), "clicked",
				G_CALLBACK (gtk_button_mute_chan_change),(gpointer) interface );

		gtk_widget_show (interface->channel[c].separator);
    	gtk_widget_show (interface->channel[c].box_vbox);
    	gtk_widget_show (interface->channel[c].box_hbox_entry_gain);
    	gtk_widget_show (interface->channel[c].box_hbox_button_stereo);
    	gtk_widget_show (interface->channel[c].box_hbox_button_mute);
    	gtk_widget_show (interface->channel[c].label_gain);
    	gtk_widget_show (interface->channel[c].label_channel_out);
    	gtk_widget_show (interface->channel[c].fader_gain);
    	gtk_widget_show (interface->channel[c].spinbutton_gain);
		/* omit for last channel */
		if( c < interface->channel_count -1){
        	gtk_widget_show (interface->channel[c].spacer_stereo);
        	gtk_widget_show (interface->channel[c].label_stereo);
    		gtk_widget_show (interface->channel[c].button_stereo);
		}
    	gtk_widget_show (interface->channel[c].spacer_mute);
    	gtk_widget_show (interface->channel[c].button_mute);
    	gtk_widget_show (interface->channel[c].label_mute);
    	gtk_widget_show (interface->channel[c].label_channel_top);
    	gtk_widget_show (interface->channel[c].label_channel_bottom);
    	gtk_widget_show (interface->channel[c].separator_top);
    	gtk_widget_show (interface->channel[c].separator_buttons);
    	gtk_widget_show (interface->channel[c].separator_bottom);

    	gtk_box_pack_start (GTK_BOX (interface->box_channels_hbox), interface->channel[c].box_vbox, FALSE, FALSE, FALSE);
		gtk_box_pack_start (GTK_BOX (interface->box_channels_hbox), interface->channel[c].separator, FALSE, FALSE, FALSE);
    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].label_channel_top, FALSE, FALSE, FALSE);
    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].separator_top, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].label_channel_bottom, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].separator_bottom, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].box_hbox_button_stereo, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].box_hbox_button_mute, FALSE, FALSE, FALSE);
    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_hbox_button_mute), interface->channel[c].spacer_mute, FALSE, FALSE, FALSE);
    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_hbox_button_mute), interface->channel[c].button_mute, FALSE, FALSE, FALSE);
    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_hbox_button_mute), interface->channel[c].label_mute, FALSE, FALSE, FALSE);
		/* omit for last channel */
		if( c < interface->channel_count -1){
    		gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_hbox_button_stereo), interface->channel[c].spacer_stereo, FALSE, FALSE, FALSE);
    		gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_hbox_button_stereo), interface->channel[c].button_stereo, FALSE, FALSE, FALSE);
    		gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_hbox_button_stereo), interface->channel[c].label_stereo, FALSE, FALSE, FALSE);
		}
		gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].separator_buttons, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].box_hbox_entry_gain, FALSE, TRUE, FALSE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_hbox_entry_gain), interface->channel[c].spinbutton_gain, TRUE, FALSE, TRUE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].fader_gain, TRUE, TRUE, TRUE);
    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].label_gain, FALSE, FALSE, FALSE);

    	if (!interface->omit_chan_pan){

    		interface->channel[c].box_hbox_knob_pan = gtk_hbox_new (FALSE, 0);
    		interface->channel[c].box_hbox_entry_pan = gtk_hbox_new (FALSE, 0);
    		interface->channel[c].label_pan = gtk_label_new("Pan");
	    	interface->channel[c].knob_pan = phat_knob_new_with_range(pan,KNOB_PAN_MIN,KNOB_PAN_MAX,KNOB_PAN_STEP);
	    	interface->channel[c].spinbutton_pan = gtk_spin_button_new_with_range(KNOB_PAN_MIN,KNOB_PAN_MAX,KNOB_PAN_STEP);

			gtk_widget_set_name (interface->channel[c].knob_pan,utf_name);
			gtk_widget_set_name (interface->channel[c].spinbutton_pan,utf_name);

			g_signal_connect (G_OBJECT (interface->channel[c].knob_pan), "value_changed",
					G_CALLBACK (gtk_knob_pan_chan_change),(gpointer) interface );

			g_signal_connect (G_OBJECT (interface->channel[c].spinbutton_pan), "value_changed",
					G_CALLBACK (gtk_spinbutton_pan_chan_change),(gpointer) interface );

			gtk_widget_set_style(interface->channel[c].label_pan,interface->small_label_style);
			gtk_widget_set_size_request (interface->channel[c].knob_pan, KNOB_SIZE+6, KNOB_SIZE+6);
	    	gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].spinbutton_pan),DEFAULT_ENTRY_LEN);
			/* FIXME this breaks the interface with size 1 ?? */
			gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].spinbutton_pan),DEFAULT_ENTRY_DIGITS);
	    	gtk_widget_set_style(interface->channel[c].spinbutton_pan,interface->spinbutton_style);

	    	gtk_widget_show (interface->channel[c].box_hbox_knob_pan);
	    	gtk_widget_show (interface->channel[c].box_hbox_entry_pan);
	    	gtk_widget_show (interface->channel[c].label_pan);
	    	gtk_widget_show (interface->channel[c].knob_pan);
	    	gtk_widget_show (interface->channel[c].spinbutton_pan);

	    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].box_hbox_entry_pan, FALSE, TRUE, FALSE);
	    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_hbox_entry_pan), interface->channel[c].spinbutton_pan, TRUE, FALSE, TRUE);
	    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].box_hbox_knob_pan, FALSE, FALSE, FALSE);
	    	gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].label_pan, FALSE, FALSE, FALSE);
	    	gtk_box_pack_start (GTK_BOX ( interface->channel[c].box_hbox_knob_pan), interface->channel[c].knob_pan, TRUE, FALSE, TRUE);
    	}

	    gtk_box_pack_end (GTK_BOX ( interface->channel[c].box_vbox), interface->channel[c].label_channel_out, FALSE, FALSE, FALSE);

		for(s=0;s<(interface->prefader_count+interface->postfader_count);s++){

			interface->channel[c].send[s].label_send = gtk_label_new(interface->output[s].label);
			interface->channel[c].send[s].box_vbox = gtk_vbox_new (FALSE, 0);
			interface->channel[c].send[s].box_hbox_upper = gtk_hbox_new (FALSE, 0);
			interface->channel[c].send[s].box_hbox_middle = gtk_hbox_new (FALSE, 0);
			interface->channel[c].send[s].box_hbox_lower = gtk_hbox_new (FALSE, 0);
			interface->channel[c].send[s].separator = gtk_hseparator_new ();
			interface->channel[c].send[s].label_out_top = gtk_label_new("Gain");


			gtk_widget_set_style(interface->channel[c].send[s].label_send,interface->medium_label_style);
			gtk_widget_set_style(interface->channel[c].send[s].label_out_top,interface->small_label_style);

			interface->channel[c].send[s].knob_gain = phat_knob_new_with_range(interface->channel[c].send[s].gain,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
			interface->channel[c].send[s].spinbutton_gain = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);

			sprintf(lc_name,"%i,%i",c+1,s+1);
			utf_name = stringprep_locale_to_utf8(lc_name);
			gtk_widget_set_name (interface->channel[c].send[s].knob_gain,utf_name);
			gtk_widget_set_name (interface->channel[c].send[s].spinbutton_gain,utf_name);

			gtk_widget_set_size_request (interface->channel[c].send[s].knob_gain, KNOB_SIZE, KNOB_SIZE);
			gtk_widget_set_style(interface->channel[c].send[s].spinbutton_gain,interface->spinbutton_style);
			gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_gain),DEFAULT_ENTRY_LEN);
			gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain),DEFAULT_ENTRY_DIGITS);

			g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_gain), "value_changed",
								G_CALLBACK (gtk_knob_gain_send_change),(gpointer) interface);

			g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_gain), "value_changed",
								G_CALLBACK (gtk_spinbutton_gain_send_change),(gpointer) interface);

			gtk_box_pack_start (GTK_BOX (interface->channel[c].box_vbox),interface->channel[c].send[s].box_vbox, FALSE, FALSE, FALSE);
			gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox),interface->channel[c].send[s].box_hbox_upper, FALSE, FALSE, FALSE);
			gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_upper),interface->channel[c].send[s].label_send, TRUE, FALSE, FALSE);

			gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox),interface->channel[c].send[s].box_hbox_middle, FALSE, FALSE, FALSE);
		    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox),interface->channel[c].send[s].box_hbox_lower, FALSE, FALSE, FALSE);
		    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox), interface->channel[c].send[s].separator, FALSE, FALSE, FALSE);

			gtk_widget_show (interface->channel[c].send[s].label_send);
			gtk_widget_show (interface->channel[c].send[s].box_vbox);
			gtk_widget_show (interface->channel[c].send[s].box_hbox_upper);
			gtk_widget_show (interface->channel[c].send[s].box_hbox_middle);
			gtk_widget_show (interface->channel[c].send[s].box_hbox_lower);
			gtk_widget_show (interface->channel[c].send[s].separator);

			if (!interface->omit_send_pan && !interface->omit_send_gain){

				interface->channel[c].send[s].box_vbox_middle_left = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_vbox_middle_right = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_knob_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_knob_right = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_entry_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_entry_right = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_vbox_lower_left = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_vbox_lower_right = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_knob_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_knob_right = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_entry_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_entry_right = gtk_hbox_new (FALSE, 0);

				interface->channel[c].send[s].label_pan = gtk_label_new("Pan");
				interface->channel[c].send[s].label_gain_l = gtk_label_new("Gain L");
				interface->channel[c].send[s].label_gain_r = gtk_label_new("Gain R");

				gtk_widget_set_style(interface->channel[c].send[s].label_pan,interface->small_label_style);
				gtk_widget_set_style(interface->channel[c].send[s].label_gain_l,interface->small_label_style);
				gtk_widget_set_style(interface->channel[c].send[s].label_gain_r,interface->small_label_style);

				interface->channel[c].send[s].knob_pan = phat_knob_new_with_range(pan,KNOB_PAN_MIN,KNOB_PAN_MAX,KNOB_PAN_STEP);
				interface->channel[c].send[s].knob_gain_l = phat_knob_new_with_range(interface->channel[c].send[s].gain_l,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
				interface->channel[c].send[s].knob_gain_r = phat_knob_new_with_range(interface->channel[c].send[s].gain_r,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);

				gtk_widget_set_size_request (interface->channel[c].send[s].knob_pan, KNOB_SIZE, KNOB_SIZE);
				gtk_widget_set_size_request (interface->channel[c].send[s].knob_gain_l, KNOB_SIZE, KNOB_SIZE);
				gtk_widget_set_size_request (interface->channel[c].send[s].knob_gain_r, KNOB_SIZE, KNOB_SIZE);

				interface->channel[c].send[s].spinbutton_pan = gtk_spin_button_new_with_range(KNOB_PAN_MIN,KNOB_PAN_MAX,KNOB_PAN_STEP);
				interface->channel[c].send[s].spinbutton_gain_l = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
				interface->channel[c].send[s].spinbutton_gain_r = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);

				gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_pan),DEFAULT_ENTRY_DIGITS);
				gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_l),DEFAULT_ENTRY_DIGITS);
				gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_r),DEFAULT_ENTRY_DIGITS);

				gtk_widget_set_style(interface->channel[c].send[s].spinbutton_pan,interface->spinbutton_style);
				gtk_widget_set_style(interface->channel[c].send[s].spinbutton_gain_l,interface->spinbutton_style);
				gtk_widget_set_style(interface->channel[c].send[s].spinbutton_gain_r,interface->spinbutton_style);

				gtk_widget_set_name (interface->channel[c].send[s].knob_pan,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].spinbutton_pan,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].knob_gain_l,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].spinbutton_gain_l,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].knob_gain_r,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].spinbutton_gain_r,utf_name);

				gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_pan),DEFAULT_ENTRY_LEN);
				gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_gain_l),DEFAULT_ENTRY_LEN);
				gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_gain_r),DEFAULT_ENTRY_LEN);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_pan), "value_changed",
					      G_CALLBACK (gtk_knob_pan_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_gain_l), "value_changed",
				      G_CALLBACK (gtk_knob_gain_l_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_gain_r), "value_changed",
				      G_CALLBACK (gtk_knob_gain_r_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_pan), "value_changed",
					      G_CALLBACK (gtk_spinbutton_pan_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_gain_l), "value_changed",
				      G_CALLBACK (gtk_spinbutton_gain_l_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_gain_r), "value_changed",
				      G_CALLBACK (gtk_spinbutton_gain_r_send_change),(gpointer) interface);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle),interface->channel[c].send[s].box_vbox_middle_left, FALSE, FALSE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle),interface->channel[c].send[s].box_vbox_middle_right, FALSE, FALSE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower),interface->channel[c].send[s].box_vbox_lower_left, FALSE, FALSE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower),interface->channel[c].send[s].box_vbox_lower_right, FALSE, FALSE, FALSE);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].label_out_top, FALSE, FALSE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_right),interface->channel[c].send[s].label_pan, FALSE, FALSE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_left),interface->channel[c].send[s].label_gain_l, FALSE, FALSE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_right),interface->channel[c].send[s].label_gain_r, FALSE, FALSE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_knob_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_right),interface->channel[c].send[s].box_hbox_middle_knob_right, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_knob_left),interface->channel[c].send[s].knob_gain, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_knob_right),interface->channel[c].send[s].knob_pan, TRUE, FALSE, TRUE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_left),interface->channel[c].send[s].box_hbox_lower_knob_left,  FALSE, TRUE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_right),interface->channel[c].send[s].box_hbox_lower_knob_right,  FALSE, TRUE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_knob_left),interface->channel[c].send[s].knob_gain_l, TRUE, FALSE, TRUE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_knob_right),interface->channel[c].send[s].knob_gain_r, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_entry_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_right),interface->channel[c].send[s].box_hbox_middle_entry_right, FALSE, TRUE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_left),interface->channel[c].send[s].box_hbox_lower_entry_left, FALSE, TRUE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_right),interface->channel[c].send[s].box_hbox_lower_entry_right, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_entry_left),interface->channel[c].send[s].spinbutton_gain, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_entry_right),interface->channel[c].send[s].spinbutton_pan, TRUE, FALSE, TRUE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_entry_left),interface->channel[c].send[s].spinbutton_gain_l, TRUE, FALSE, TRUE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_entry_right),interface->channel[c].send[s].spinbutton_gain_r, TRUE, FALSE, TRUE);

				gtk_widget_show (interface->channel[c].send[s].box_vbox_middle_left);
				gtk_widget_show (interface->channel[c].send[s].box_vbox_middle_right);
				gtk_widget_show (interface->channel[c].send[s].box_vbox_lower_left);
				gtk_widget_show (interface->channel[c].send[s].box_vbox_lower_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_knob_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_knob_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_entry_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_entry_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_knob_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_knob_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_entry_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_entry_right);

			    gtk_widget_show (interface->channel[c].send[s].label_pan);
		    	gtk_widget_show (interface->channel[c].send[s].label_gain_l);
		    	gtk_widget_show (interface->channel[c].send[s].label_gain_r);
			    gtk_widget_show (interface->channel[c].send[s].knob_pan);
		    	gtk_widget_show (interface->channel[c].send[s].knob_gain_l);
		    	gtk_widget_show (interface->channel[c].send[s].knob_gain_r);
			    gtk_widget_show (interface->channel[c].send[s].spinbutton_pan);
		    	gtk_widget_show (interface->channel[c].send[s].spinbutton_gain_l);
		    	gtk_widget_show (interface->channel[c].send[s].spinbutton_gain_r);

			}

			if (interface->omit_send_pan && !interface->omit_send_gain){

				interface->channel[c].send[s].box_vbox_middle_left = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_knob_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_entry_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_vbox_lower_left = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_vbox_lower_right = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_knob_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_knob_right = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_entry_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_lower_entry_right = gtk_hbox_new (FALSE, 0);

				interface->channel[c].send[s].label_gain_l = gtk_label_new("Gain L");
				interface->channel[c].send[s].label_gain_r = gtk_label_new("Gain R");

				gtk_widget_set_style(interface->channel[c].send[s].label_gain_l,interface->small_label_style);
				gtk_widget_set_style(interface->channel[c].send[s].label_gain_r,interface->small_label_style);


				interface->channel[c].send[s].knob_gain_l = phat_knob_new_with_range(interface->channel[c].send[s].gain_l,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
				interface->channel[c].send[s].knob_gain_r = phat_knob_new_with_range(interface->channel[c].send[s].gain_r,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
				interface->channel[c].send[s].spinbutton_gain_l = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
				interface->channel[c].send[s].spinbutton_gain_r = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);


				gtk_widget_set_name (interface->channel[c].send[s].knob_gain_l,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].spinbutton_gain_l,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].knob_gain_r,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].spinbutton_gain_r,utf_name);

				gtk_widget_set_size_request (interface->channel[c].send[s].knob_gain_l, KNOB_SIZE, KNOB_SIZE);
				gtk_widget_set_size_request (interface->channel[c].send[s].knob_gain_r, KNOB_SIZE, KNOB_SIZE);
				gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_gain_l),DEFAULT_ENTRY_LEN);
				gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_gain_r),DEFAULT_ENTRY_LEN);
				gtk_widget_set_style(interface->channel[c].send[s].spinbutton_gain_l,interface->spinbutton_style);
				gtk_widget_set_style(interface->channel[c].send[s].spinbutton_gain_r,interface->spinbutton_style);
				gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_l),DEFAULT_ENTRY_DIGITS);
				gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_r),DEFAULT_ENTRY_DIGITS);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_gain_l), "value_changed",
				      G_CALLBACK (gtk_knob_gain_l_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_gain_r), "value_changed",
				      G_CALLBACK (gtk_knob_gain_r_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_gain_l), "value_changed",
				      G_CALLBACK (gtk_spinbutton_gain_l_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_gain_r), "value_changed",
				      G_CALLBACK (gtk_spinbutton_gain_r_send_change),(gpointer) interface);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle),interface->channel[c].send[s].box_vbox_middle_left, TRUE, FALSE, TRUE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower),interface->channel[c].send[s].box_vbox_lower_left, FALSE, FALSE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower),interface->channel[c].send[s].box_vbox_lower_right, FALSE, FALSE, FALSE);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].label_out_top, FALSE, FALSE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_left),interface->channel[c].send[s].label_gain_l, FALSE, FALSE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_right),interface->channel[c].send[s].label_gain_r, FALSE, FALSE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_knob_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_knob_left),interface->channel[c].send[s].knob_gain, TRUE, FALSE, TRUE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_left),interface->channel[c].send[s].box_hbox_lower_knob_left,  FALSE, TRUE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_right),interface->channel[c].send[s].box_hbox_lower_knob_right,  FALSE, TRUE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_knob_left),interface->channel[c].send[s].knob_gain_l, TRUE, FALSE, TRUE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_knob_right),interface->channel[c].send[s].knob_gain_r, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_entry_left, FALSE, TRUE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_left),interface->channel[c].send[s].box_hbox_lower_entry_left, FALSE, TRUE, FALSE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_lower_right),interface->channel[c].send[s].box_hbox_lower_entry_right, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_entry_left),interface->channel[c].send[s].spinbutton_gain, TRUE, FALSE, TRUE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_entry_left),interface->channel[c].send[s].spinbutton_gain_l, TRUE, FALSE, TRUE);
		    	gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_lower_entry_right),interface->channel[c].send[s].spinbutton_gain_r, TRUE, FALSE, TRUE);

				gtk_widget_show (interface->channel[c].send[s].box_vbox_middle_left);
				gtk_widget_show (interface->channel[c].send[s].box_vbox_lower_left);
				gtk_widget_show (interface->channel[c].send[s].box_vbox_lower_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_knob_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_entry_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_knob_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_knob_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_entry_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_lower_entry_right);

		    	gtk_widget_show (interface->channel[c].send[s].label_gain_l);
		    	gtk_widget_show (interface->channel[c].send[s].label_gain_r);
		    	gtk_widget_show (interface->channel[c].send[s].knob_gain_l);
		    	gtk_widget_show (interface->channel[c].send[s].knob_gain_r);
		    	gtk_widget_show (interface->channel[c].send[s].spinbutton_gain_l);
		    	gtk_widget_show (interface->channel[c].send[s].spinbutton_gain_r);

			}

			if (!interface->omit_send_pan && interface->omit_send_gain){

				interface->channel[c].send[s].box_vbox_middle_left = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_vbox_middle_right = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_knob_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_knob_right = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_entry_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_entry_right = gtk_hbox_new (FALSE, 0);

				interface->channel[c].send[s].label_pan = gtk_label_new("Pan");
				gtk_widget_set_style(interface->channel[c].send[s].label_pan,interface->small_label_style);


				if (interface->channel[c].send[s].gain_l == 0 && interface->channel[c].send[s].gain_r < 0){
					pan = interface->channel[c].send[s].gain_r;
				} else if (interface->channel[c].send[s].gain_r == 0 && interface->channel[c].send[s].gain_l < 0){
					pan = interface->channel[c].send[s].gain_l * (-1);
				} else {
					pan = 0;
				}

				interface->channel[c].send[s].knob_pan = phat_knob_new_with_range(pan,KNOB_PAN_MIN,KNOB_PAN_MAX,KNOB_PAN_STEP);
				interface->channel[c].send[s].spinbutton_pan = gtk_spin_button_new_with_range(KNOB_PAN_MIN,KNOB_PAN_MAX,KNOB_PAN_STEP);

				gtk_widget_set_style(interface->channel[c].send[s].spinbutton_pan,interface->spinbutton_style);
				gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_pan),DEFAULT_ENTRY_DIGITS);
				gtk_widget_set_size_request (interface->channel[c].send[s].knob_pan, KNOB_SIZE, KNOB_SIZE);
				gtk_widget_set_name (interface->channel[c].send[s].knob_pan,utf_name);
				gtk_widget_set_name (interface->channel[c].send[s].spinbutton_pan,utf_name);


				gtk_entry_set_width_chars(GTK_ENTRY(interface->channel[c].send[s].spinbutton_pan),DEFAULT_ENTRY_LEN);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].knob_pan), "value_changed",
					      G_CALLBACK (gtk_knob_pan_send_change),(gpointer) interface);

			    g_signal_connect (G_OBJECT (interface->channel[c].send[s].spinbutton_pan), "value_changed",
					      G_CALLBACK (gtk_spinbutton_pan_send_change),(gpointer) interface);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle),interface->channel[c].send[s].box_vbox_middle_left, FALSE, FALSE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle),interface->channel[c].send[s].box_vbox_middle_right, FALSE, FALSE, FALSE);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].label_out_top, FALSE, FALSE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_right),interface->channel[c].send[s].label_pan, FALSE, FALSE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_knob_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_right),interface->channel[c].send[s].box_hbox_middle_knob_right, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_knob_left),interface->channel[c].send[s].knob_gain, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_knob_right),interface->channel[c].send[s].knob_pan, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_entry_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_right),interface->channel[c].send[s].box_hbox_middle_entry_right, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_entry_left),interface->channel[c].send[s].spinbutton_gain, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_entry_right),interface->channel[c].send[s].spinbutton_pan, TRUE, FALSE, TRUE);

				gtk_widget_show (interface->channel[c].send[s].box_vbox_middle_left);
				gtk_widget_show (interface->channel[c].send[s].box_vbox_middle_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_knob_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_knob_right);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_entry_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_entry_right);

			    gtk_widget_show (interface->channel[c].send[s].label_pan);
			    gtk_widget_show (interface->channel[c].send[s].knob_pan);
			    gtk_widget_show (interface->channel[c].send[s].spinbutton_pan);

			}

			if (interface->omit_send_pan && interface->omit_send_gain){

				interface->channel[c].send[s].box_vbox_middle_left = gtk_vbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_knob_left = gtk_hbox_new (FALSE, 0);
				interface->channel[c].send[s].box_hbox_middle_entry_left = gtk_hbox_new (FALSE, 0);

			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle),interface->channel[c].send[s].box_vbox_middle_left, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].label_out_top, FALSE, FALSE, FALSE);
				gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_knob_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_knob_left),interface->channel[c].send[s].knob_gain, TRUE, FALSE, TRUE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_vbox_middle_left),interface->channel[c].send[s].box_hbox_middle_entry_left, FALSE, TRUE, FALSE);
			    gtk_box_pack_start (GTK_BOX (interface->channel[c].send[s].box_hbox_middle_entry_left),interface->channel[c].send[s].spinbutton_gain, TRUE, FALSE, TRUE);

				gtk_widget_show (interface->channel[c].send[s].box_vbox_middle_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_knob_left);
				gtk_widget_show (interface->channel[c].send[s].box_hbox_middle_entry_left);

			}
			gtk_widget_show (interface->channel[c].send[s].label_out_top);
		    gtk_widget_show (interface->channel[c].send[s].knob_gain);
		    gtk_widget_show (interface->channel[c].send[s].spinbutton_gain);

		}
    }

    gtk_widget_show (interface->window_channels);
    gtk_widget_show (interface->window_channels_top);

	g_free(utf_name);

	return 0;
}

static gint interface_init_gtk_outputs(jmd_interface_t *interface)
{
	gint s=0;
	gchar window_title[32];
	gchar label[32];
	gchar lc_name[5];
	gchar *utf_name=NULL;
	GtkObject *hadj;

    gfloat upper = (gfloat) (interface->prefader_count + interface->postfader_count - 8);

	hadj = gtk_adjustment_new(upper, 0.0, upper, 0.2,0.2 , 0.2);
    interface->window_outputs = gtk_scrolled_window_new (GTK_ADJUSTMENT(hadj), NULL);

    interface->window_outputs_top = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	interface->box_outputs_hbox = gtk_hbox_new (FALSE, 0);

	g_signal_connect_swapped (G_OBJECT (interface->window_outputs_top), "delete_event",
			      G_CALLBACK (gtk_toggle_widget_view),
                              G_OBJECT (interface->window_outputs_top));

	g_signal_connect_swapped (G_OBJECT (interface->window_outputs_top), "destroy",
			      G_CALLBACK (gtk_toggle_widget_view),
                              G_OBJECT (interface->window_outputs_top));

	sprintf( window_title,"JackMixDesk Outs 1- %i",interface->prefader_count+interface->postfader_count);
	gtk_window_set_title (GTK_WINDOW (interface->window_outputs_top), window_title);

    gtk_container_add (GTK_CONTAINER (interface->window_outputs_top),interface->window_outputs);

	gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (interface->window_outputs), interface->box_outputs_hbox);

    gtk_container_set_border_width (GTK_CONTAINER (interface->window_outputs_top), 0);
    gtk_container_set_border_width (GTK_CONTAINER (interface->window_outputs), 0);

	gtk_widget_set_size_request(GTK_WIDGET(interface->window_outputs_top),
									OUTPUT_WIDTH + SEPARATOR_WIDTH + OUTPUT_OFFSET, 0);

	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (interface->window_outputs),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_NEVER);


    if (interface->prefader_count + interface->postfader_count < 7) {
    	gtk_window_set_default_size (GTK_WINDOW(interface->window_outputs_top),
    								(interface->prefader_count + interface->postfader_count) * (OUTPUT_WIDTH + SEPARATOR_WIDTH) + OUTPUT_OFFSET, 580);
    } else {
    	gtk_window_set_default_size (GTK_WINDOW(interface->window_outputs_top),
    								8 * (OUTPUT_WIDTH + SEPARATOR_WIDTH) + OUTPUT_OFFSET, 580);
    }

    gtk_widget_show (interface->box_outputs_hbox);

	/* create the widgets for the output window */
    for(s=0;s<(interface->prefader_count+interface->postfader_count);s++){

    	sprintf(label,"%s",interface->output[s].label);

    	interface->output[s].separator =  gtk_vseparator_new ();
   		gtk_widget_set_size_request(interface->output[s].separator,SEPARATOR_WIDTH, -1);
       	interface->output[s].box_vbox = gtk_vbox_new (FALSE, 0);
   		gtk_widget_set_size_request(interface->output[s].box_vbox,OUTPUT_WIDTH, -1);
    	interface->output[s].separator_top =  gtk_hseparator_new ();
    	interface->output[s].separator_bottom =  gtk_hseparator_new ();
    	interface->output[s].separator_buttons =  gtk_hseparator_new ();
    	interface->output[s].box_hbox_label_top = gtk_hbox_new (FALSE, 0);
    	interface->output[s].box_hbox_label_bottom = gtk_hbox_new (FALSE, 0);
    	interface->output[s].label_out_top = gtk_label_new(label);
    	interface->output[s].label_out_bottom = gtk_label_new(label);
    	interface->output[s].label_gain = gtk_label_new("Gain");
    	gtk_widget_set_style(interface->output[s].label_gain,interface->small_label_style);
    	interface->output[s].box_hbox_fader = gtk_hbox_new (FALSE, 0);
    	interface->output[s].fader_gain = phat_vfan_slider_new_with_range(interface->output[s].gain,KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
    	interface->output[s].box_hbox_entry = gtk_hbox_new (FALSE, 0);
    	interface->output[s].spinbutton_gain = gtk_spin_button_new_with_range(KNOB_GAIN_MIN,KNOB_GAIN_MAX,KNOB_GAIN_STEP);
		gtk_widget_set_style(interface->output[s].spinbutton_gain,interface->spinbutton_style);
		gtk_spin_button_set_digits(GTK_SPIN_BUTTON(interface->output[s].spinbutton_gain),DEFAULT_ENTRY_DIGITS);
		gtk_widget_set_style(interface->output[s].spinbutton_gain,interface->spinbutton_style);
		interface->output[s].box_hbox_mute = gtk_hbox_new (FALSE, 0);
		interface->output[s].box_hbox_link = gtk_hbox_new (FALSE, 0);
		interface->output[s].spacer_mute = gtk_hbox_new (FALSE, 0);
		interface->output[s].button_mute = gtk_toggle_button_new();
		interface->output[s].label_mute = gtk_label_new("Mute");
		gtk_box_set_spacing(GTK_BOX(interface->output[s].box_vbox),BOX_VSPACING);
		gtk_box_set_spacing(GTK_BOX(interface->output[s].box_hbox_mute),BOX_HSPACING);
		gtk_box_set_spacing(GTK_BOX(interface->output[s].box_hbox_link),BOX_HSPACING);
		gtk_widget_set_style(interface->output[s].label_mute,interface->small_label_style);
    	gtk_widget_set_size_request(interface->output[s].box_hbox_link,-1,BUTTON_SIZE);
    	gtk_widget_set_size_request(interface->output[s].button_mute,BUTTON_SIZE,BUTTON_SIZE);

		if( s < interface->prefader_count + interface->postfader_count -1){
			interface->output[s].spacer_link = gtk_hbox_new (FALSE, 0);
    		interface->output[s].label_link = gtk_label_new("Link");
    		gtk_widget_set_style(interface->output[s].label_link,interface->small_label_style);
     		interface->output[s].button_link = gtk_toggle_button_new();
     		gtk_widget_set_size_request(interface->output[s].button_link,BUTTON_SIZE,BUTTON_SIZE);
			g_signal_connect (G_OBJECT (interface->output[s].button_link), "clicked",
					G_CALLBACK (gtk_button_link_out_change),(gpointer) interface );
		}

    	gtk_entry_set_width_chars(GTK_ENTRY(interface->output[s].spinbutton_gain),DEFAULT_ENTRY_LEN);

    	sprintf(lc_name,"%i",s+1);
		utf_name = stringprep_locale_to_utf8(lc_name);
		gtk_widget_set_name (interface->output[s].fader_gain,utf_name);
		gtk_widget_set_name (interface->output[s].spinbutton_gain,utf_name);
		gtk_widget_set_name (interface->output[s].button_mute,utf_name);
		if( s < interface->prefader_count + interface->postfader_count -1){
			gtk_widget_set_name (interface->output[s].button_link,utf_name);
		}

		g_signal_connect (G_OBJECT (interface->output[s].fader_gain), "value_changed",
					G_CALLBACK (gtk_fader_gain_out_change),(gpointer) interface );

		g_signal_connect (G_OBJECT (interface->output[s].spinbutton_gain), "value_changed",
					G_CALLBACK (gtk_spinbutton_gain_out_change),(gpointer) interface );

		g_signal_connect (G_OBJECT (interface->output[s].button_mute), "clicked",
					G_CALLBACK (gtk_button_mute_out_change),(gpointer) interface );


    	gtk_box_pack_start (GTK_BOX (interface->box_outputs_hbox),interface->output[s].box_vbox, FALSE, FALSE, FALSE);
		gtk_box_pack_start (GTK_BOX (interface->box_outputs_hbox),interface->output[s].separator, FALSE, FALSE, FALSE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_vbox),interface->output[s].box_hbox_label_top, FALSE, TRUE, FALSE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_vbox),interface->output[s].separator_top, FALSE, TRUE, FALSE);
    	gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].box_hbox_label_bottom, FALSE, TRUE, FALSE);
    	gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].separator_bottom, FALSE, TRUE, FALSE);
		gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].box_hbox_link, FALSE, FALSE, FALSE);
		gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].box_hbox_mute, FALSE, FALSE, FALSE);
		gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].separator_buttons, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].box_hbox_entry, FALSE, TRUE, FALSE);
    	gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].box_hbox_fader, TRUE, TRUE, TRUE);
    	gtk_box_pack_end (GTK_BOX (interface->output[s].box_vbox),interface->output[s].label_gain, FALSE, FALSE, FALSE);
    	gtk_box_pack_end (GTK_BOX (interface->output[s].box_hbox_fader),interface->output[s].fader_gain, TRUE, FALSE, TRUE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_hbox_label_top),interface->output[s].label_out_top, TRUE, FALSE, TRUE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_hbox_label_bottom),interface->output[s].label_out_bottom, TRUE, FALSE, TRUE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_hbox_mute),interface->output[s].spacer_mute, FALSE,FALSE,FALSE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_hbox_mute),interface->output[s].button_mute, FALSE,FALSE,FALSE);
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_hbox_mute),interface->output[s].label_mute, FALSE,FALSE,FALSE);
    	if( s < interface->prefader_count + interface->postfader_count -1){
    		gtk_box_pack_end (GTK_BOX (interface->output[s].box_hbox_link),interface->output[s].spacer_link, FALSE,FALSE,FALSE);
    		gtk_box_pack_end (GTK_BOX (interface->output[s].box_hbox_link),interface->output[s].button_link, FALSE,FALSE,FALSE);
    		gtk_box_pack_end (GTK_BOX (interface->output[s].box_hbox_link),interface->output[s].label_link, FALSE,FALSE,FALSE);
    	}
    	gtk_box_pack_start (GTK_BOX (interface->output[s].box_hbox_entry),interface->output[s].spinbutton_gain, TRUE, FALSE, TRUE);

    	gtk_widget_show (interface->output[s].separator);
    	gtk_widget_show (interface->output[s].box_vbox);
    	gtk_widget_show (interface->output[s].separator_top);
    	gtk_widget_show (interface->output[s].separator_bottom);
    	gtk_widget_show (interface->output[s].separator_buttons);
    	gtk_widget_show (interface->output[s].box_hbox_label_top);
    	gtk_widget_show (interface->output[s].box_hbox_label_bottom);
    	gtk_widget_show (interface->output[s].label_out_top);
    	gtk_widget_show (interface->output[s].label_out_bottom);
    	gtk_widget_show (interface->output[s].label_gain);
    	gtk_widget_show (interface->output[s].box_hbox_fader);
    	gtk_widget_show (interface->output[s].fader_gain);
    	gtk_widget_show (interface->output[s].box_hbox_entry);
    	gtk_widget_show (interface->output[s].spinbutton_gain);
    	gtk_widget_show (interface->output[s].box_hbox_mute);
    	gtk_widget_show (interface->output[s].spacer_mute);
    	gtk_widget_show (interface->output[s].button_mute);
    	gtk_widget_show (interface->output[s].label_mute);
    	gtk_widget_show (interface->output[s].box_hbox_link);

    	if( s < interface->prefader_count + interface->postfader_count -1){
    		gtk_widget_show (interface->output[s].spacer_link);
    		gtk_widget_show (interface->output[s].label_link);
    		gtk_widget_show (interface->output[s].button_link);
    	}
    }

    gtk_widget_show (interface->window_outputs);
    gtk_widget_show (interface->window_outputs_top);

    g_free(utf_name);

	return 0;
}

static gint interface_update_gtk_channels(jmd_interface_t *interface)
{
	gint c,s;
	gfloat pan;

	/* update the widgets for the channel window */
    for(c=0;c<interface->channel_count;c++){

		if (interface->channel[c].send[0].gain_l == 0 && interface->channel[c].send[0].gain_r < 0){
			pan = interface->channel[c].send[0].gain_r;
		} else if (interface->channel[c].send[0].gain_r == 0 && interface->channel[c].send[0].gain_l < 0){
			pan = interface->channel[c].send[0].gain_l * (-1);
		} else {
			pan = 0;
		}

    	phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->channel[c].fader_gain),interface->channel[c].send[0].gain);
	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].spinbutton_gain),interface->channel[c].send[0].gain);

    	if (!interface->omit_chan_pan){
	    	phat_knob_set_value(PHAT_KNOB (interface->channel[c].knob_pan),pan);
	    	gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].spinbutton_pan),pan);
    	}
		/* omit for last channel */
		if( c < interface->channel_count -1){
			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(interface->channel[c].button_stereo), interface->channel[c].button_stereo_state);
		}
     	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(interface->channel[c].button_mute),interface->channel[c].button_mute_state);

		for(s=0;s<(interface->prefader_count+interface->postfader_count);s++){

			phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_gain),interface->channel[c].send[s].gain);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain),interface->channel[c].send[s].gain);

			if (!interface->omit_send_pan && !interface->omit_send_gain){
				if (interface->channel[c].send[s].gain_l == 0 && interface->channel[c].send[s].gain_r < 0){
					pan = interface->channel[c].send[s].gain_r;
				} else if (interface->channel[c].send[s].gain_r == 0 && interface->channel[c].send[s].gain_l < 0){
					pan = interface->channel[c].send[s].gain_l * (-1);
				} else {
					pan = 0;
				}
				phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_pan),pan);
				phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_gain_l),interface->channel[c].send[s].gain_l);
				phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_gain_r),interface->channel[c].send[s].gain_r);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_pan),pan);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_l),interface->channel[c].send[s].gain_l);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_r),interface->channel[c].send[s].gain_r);

			}

			if (interface->omit_send_pan && !interface->omit_send_gain){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_gain_l),interface->channel[c].send[s].gain_l);
				phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_gain_r),interface->channel[c].send[s].gain_r);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_l),interface->channel[c].send[s].gain_l);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_gain_r),interface->channel[c].send[s].gain_r);
			}

			if (!interface->omit_send_pan && interface->omit_send_gain){
				phat_knob_set_value(PHAT_KNOB (interface->channel[c].send[s].knob_pan),pan);
				gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->channel[c].send[s].spinbutton_pan),pan);
			}
		}
    }
	return 0;
}

static gint interface_update_gtk_outputs(jmd_interface_t *interface)
{
	gint s;

	/* update the widgets for the output window */
    for(s=0;s<(interface->prefader_count+interface->postfader_count);s++){

		phat_fan_slider_set_value(PHAT_FAN_SLIDER(interface->output[s].fader_gain),interface->output[s].gain);
    	gtk_spin_button_set_value(GTK_SPIN_BUTTON(interface->output[s].spinbutton_gain),interface->output[s].gain);
     	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(interface->output[s].button_mute), interface->output[s].button_mute_state);

		if( s < interface->prefader_count + interface->postfader_count -1){
     		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(interface->output[s].button_link), interface->output[s].button_link_state);

		}
	}
	return 0;
}

static gint interface_init_gtk_menu(jmd_interface_t *interface)
{
	gchar window_title[32];

    interface->menu.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_resizable(GTK_WINDOW(interface->menu.window),FALSE);
    gtk_window_set_keep_above(GTK_WINDOW(interface->menu.window),TRUE);
    interface->menu.keep_above = 1;

    interface->menu.menu_bar = gtk_menu_bar_new ();

    interface->menu.file_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.file_item = gtk_menu_item_new_with_mnemonic("_File");
    interface->menu.quit_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_QUIT, NULL);

    interface->menu.edit_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.edit_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_EDIT, NULL);
    /* switch off menuicon */
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(interface->menu.edit_item),NULL);
    interface->menu.add_port_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.del_port_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.add_port_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ADD, NULL);
    interface->menu.del_port_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_REMOVE, NULL);
    interface->menu.add_in_port_item = gtk_image_menu_item_new_with_mnemonic ("Midi _In");

    interface->menu.add_out_port_item = gtk_menu_item_new_with_mnemonic ("Midi _Out");
    interface->menu.del_in_port_item = gtk_menu_item_new_with_mnemonic ("Midi _In");
    interface->menu.del_out_port_item = gtk_menu_item_new_with_mnemonic ("Midi _Out");

    interface->menu.view_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.view_item = gtk_menu_item_new_with_mnemonic ("_View");
    interface->menu.midimix_item = gtk_menu_item_new_with_mnemonic ("Add _Midimixer");
    interface->menu.keyboard_item = gtk_menu_item_new_with_mnemonic ("Add _Keyboard");
	interface->menu.phatpad_item = gtk_menu_item_new_with_mnemonic ("Add _Phatpad");

    interface->menu.window_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.window_item = gtk_menu_item_new_with_mnemonic("_Windows");
    interface->menu.window_tearoff_item = gtk_tearoff_menu_item_new();
	interface->menu.channel_show_item = gtk_check_menu_item_new_with_mnemonic("Show/Hide _Channel window");
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(interface->menu.channel_show_item),TRUE);
	interface->menu.output_show_item = gtk_check_menu_item_new_with_mnemonic("Show/Hide _Output window");
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(interface->menu.output_show_item),TRUE);
	interface->menu.menu_above_separator = gtk_separator_menu_item_new();
	interface->menu.menu_above_item = gtk_check_menu_item_new_with_mnemonic("Set/Unset _Menu above");
	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(interface->menu.menu_above_item),TRUE);
    interface->menu.help_menu = gtk_menu_new ();    /* Don't need to show menus */
    interface->menu.help_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP, NULL);
    gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(interface->menu.help_item),NULL);
    interface->menu.help_view_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_HELP, NULL);
	interface->menu.about_separator = gtk_separator_menu_item_new();
    interface->menu.about_item = gtk_image_menu_item_new_from_stock(GTK_STOCK_ABOUT, NULL);
	gtk_widget_set_size_request(interface->menu.about_item,-1,5);
    /* Add them to the menu */
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.file_menu), interface->menu.quit_item);

    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.edit_menu), interface->menu.add_port_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.edit_menu), interface->menu.del_port_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.add_port_menu), interface->menu.add_in_port_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.add_port_menu), interface->menu.add_out_port_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.del_port_menu), interface->menu.del_in_port_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.del_port_menu), interface->menu.del_out_port_item);

    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.view_menu), interface->menu.midimix_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.view_menu), interface->menu.keyboard_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.view_menu), interface->menu.phatpad_item);

    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.window_menu), interface->menu.window_tearoff_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.window_menu), interface->menu.channel_show_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.window_menu), interface->menu.output_show_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.window_menu), interface->menu.menu_above_separator);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.window_menu), interface->menu.menu_above_item);

    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.help_menu), interface->menu.help_view_item);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.help_menu), interface->menu.about_separator);
    gtk_menu_shell_append (GTK_MENU_SHELL (interface->menu.help_menu), interface->menu.about_item);


	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.file_item), interface->menu.file_menu);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.edit_item), interface->menu.edit_menu);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.view_item), interface->menu.view_menu);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.window_item), interface->menu.window_menu);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.help_item), interface->menu.help_menu);

	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.add_port_item), interface->menu.add_port_menu);
	gtk_menu_item_set_submenu (GTK_MENU_ITEM (interface->menu.del_port_item), interface->menu.del_port_menu);

	gtk_menu_bar_append (GTK_MENU_BAR (interface->menu.menu_bar), interface->menu.file_item);
	gtk_menu_bar_append (GTK_MENU_BAR (interface->menu.menu_bar), interface->menu.edit_item);
	gtk_menu_bar_append (GTK_MENU_BAR (interface->menu.menu_bar), interface->menu.view_item);
	gtk_menu_bar_append (GTK_MENU_BAR (interface->menu.menu_bar), interface->menu.window_item);
	gtk_menu_bar_append (GTK_MENU_BAR (interface->menu.menu_bar), interface->menu.help_item);

	g_signal_connect (G_OBJECT (interface->menu.window), "delete_event",
		      G_CALLBACK (gtk_delete), NULL);

    g_signal_connect (G_OBJECT (interface->menu.window), "destroy",
		      G_CALLBACK (gtk_destroy), NULL);
    /* Attach the callback functions to the activate signal */
    /* We can attach the Quit menu item to our exit function */

    g_signal_connect_swapped (G_OBJECT (interface->menu.quit_item), "activate",
                              G_CALLBACK (exit),
                              (gpointer) "file.quit");

    g_signal_connect_swapped (G_OBJECT (interface->menu.quit_item), "activate",
                              G_CALLBACK (gtk_destroy),
                              (gpointer) interface);

    g_signal_connect (G_OBJECT (interface->menu.about_item), "activate",
                              G_CALLBACK (gtk_about_dialog),
                              (gpointer) interface );

    g_signal_connect (G_OBJECT (interface->menu.help_view_item), "activate",
                              G_CALLBACK (gtk_menu_help),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.midimix_item), "activate",
                              G_CALLBACK (gtk_menu_midimix),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.keyboard_item), "activate",
                              G_CALLBACK (gtk_menu_keyboard),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.phatpad_item), "activate",
                              G_CALLBACK (gtk_menu_phatpad),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.add_in_port_item), "activate",
                              G_CALLBACK (gtk_menu_add_midi_in),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.add_out_port_item), "activate",
                              G_CALLBACK (gtk_menu_add_midi_out),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.del_in_port_item), "activate",
                              G_CALLBACK (gtk_menu_del_midi_in),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.del_out_port_item), "activate",
                              G_CALLBACK (gtk_menu_del_midi_out),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.window_item ), "activate",
                              G_CALLBACK (gtk_menu_update_window_item),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.channel_show_item), "activate",
                              G_CALLBACK (gtk_toggle_window_channels),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.output_show_item), "activate",
                              G_CALLBACK (gtk_toggle_window_outputs),
                              (gpointer) interface );

	g_signal_connect (G_OBJECT (interface->menu.menu_above_item), "activate",
                              G_CALLBACK (gtk_menu_menu_above),
                              (gpointer) interface );

	sprintf( window_title,"JackMixDesk Main");
	gtk_window_set_title (GTK_WINDOW (interface->menu.window), window_title);

	gtk_container_add (GTK_CONTAINER (interface->menu.window),interface->menu.menu_bar);

    gtk_container_set_border_width (GTK_CONTAINER (interface->menu.window), 0);

    /* We do need to show menu items */
    gtk_widget_show (interface->menu.menu_bar);
    gtk_widget_show (interface->menu.file_item);
    gtk_widget_show (interface->menu.quit_item);
    gtk_widget_show (interface->menu.edit_item);
    gtk_widget_show (interface->menu.add_port_item);
    gtk_widget_show (interface->menu.del_port_item);
    gtk_widget_show (interface->menu.add_in_port_item);
    gtk_widget_show (interface->menu.add_out_port_item);
    gtk_widget_show (interface->menu.del_in_port_item);
    gtk_widget_show (interface->menu.del_out_port_item);
    gtk_widget_show (interface->menu.view_item);
    gtk_widget_show (interface->menu.midimix_item);
    gtk_widget_show (interface->menu.keyboard_item);

	if (interface->alive) {
        gtk_widget_show (interface->menu.phatpad_item);
    	gtk_widget_show (interface->menu.channel_show_item);
    	gtk_widget_show (interface->menu.output_show_item);
    }
    gtk_widget_show (interface->menu.window_item);
    gtk_widget_show (interface->menu.window_tearoff_item);
    gtk_widget_show (interface->menu.menu_above_separator);
    gtk_widget_show (interface->menu.menu_above_item);
    gtk_widget_show (interface->menu.help_item);
    gtk_widget_show (interface->menu.help_view_item);
    gtk_widget_show (interface->menu.about_separator);
    gtk_widget_show (interface->menu.about_item);

	/* Window for the menu no menu! */
    gtk_widget_show (interface->menu.window);

	return 0;
}

static void interface_finish(jmd_interface_t *interface)
{
	g_free(interface->channel);
	g_free(interface->output);
	g_free(interface);

	if (interface->verbose) g_print( "Closed gtk interface.\n");
}

static gint osc_generic_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
	/* catch any incoming messages and display them. returning 1 means that the
	 * message has not been fully handled and the server should try other methods */
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		if (!quiet) {
	    	g_print("\n%s received unknown OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 1;
}

static gint osc_set_return_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
	/* handle all set returns */
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		if (interface->verbose) {
	    	g_print("\n%s received set return OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_count_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel_count=argv[0]->i;
		if (interface->verbose>1) {
	    	g_print("\n%s received channel count OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
			g_print("arg '%c' ", types[0]);
			lo_arg_pp(types[0], argv[0]);
			g_print("\n");
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_prefader_count_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->prefader_count=argv[0]->i;
		if (interface->verbose>1) {
	    	g_print("\n%s received prefader count OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
			g_print("arg '%c' ", types[0]);
			lo_arg_pp(types[0], argv[0]);
			g_print("\n");
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_postfader_count_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->postfader_count=argv[0]->i;
		if (interface->verbose>1) {
	    	g_print("\n%s received postfader count OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
			g_print("arg '%c' ", types[0]);
			lo_arg_pp(types[0], argv[0]);
			g_print("\n");
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_label_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		snprintf( interface->channel[(argv[0]->i)-1].label, DEFAULT_LABEL_LEN,"%s",argv[1]->m );
		if (interface->verbose) {
	    	g_print("\n%s received channel label OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_mute_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel[(argv[0]->i)-1].button_mute_state=argv[1]->i;
		if (interface->verbose) {
	    	g_print("\n%s received channel mute OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_stereo_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel[(argv[0]->i)-1].button_stereo_state=argv[1]->i;
		if (interface->verbose) {
	    	g_print("\n%s received channel stereo OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_send_gain_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel[(argv[0]->i)-1].send[(argv[1]->i)-1].gain=argv[2]->f;
		if (interface->verbose>1) {
	    	g_print("\n%s received channel send gain OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_send_pan_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel[(argv[0]->i)-1].send[(argv[1]->i)-1].pan=argv[2]->f;
		if (interface->verbose>1) {
	    	g_print("\n%s received channel send pan OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_send_gain_l_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel[(argv[0]->i)-1].send[(argv[1]->i)-1].gain_l=argv[2]->f;
		if (interface->verbose>1) {
	    	g_print("\n%s received channel send gain left OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_channel_send_gain_r_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->channel[(argv[0]->i)-1].send[(argv[1]->i)-1].gain_r=argv[2]->f;
		if (interface->verbose>1) {
	    	g_print("\n%s received channel send gain right OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_output_label_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		snprintf( interface->output[(argv[0]->i)-1].label, DEFAULT_LABEL_LEN,"%s",argv[1]->m );
		if (interface->verbose) {
	    	g_print("\n%s received output label OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_output_gain_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->output[(argv[0]->i)-1].gain=argv[1]->f;
		if ((interface->verbose)>1) {
	    	g_print("\n%s received output gain OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_output_mute_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->output[(argv[0]->i)-1].button_mute_state=argv[1]->i;
		if (interface->verbose) {
	    	g_print("\n%s received output mute OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_output_link_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->output[(argv[0]->i)-1].button_link_state=argv[1]->i;
		if (interface->verbose) {
	    	g_print("\n%s received output link OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static gint osc_pong_handler(const gchar *path, const gchar *types, lo_arg **argv,
			gint argc, void *data, void *user_data)
{
    gint i;
    jmd_interface_t *interface = (jmd_interface_t*)user_data;
	if (data) {
		interface->alive = 1;
		if (interface->verbose) {
	    	g_print("\n%s received pong OSC message.\n",interface->client_name);
		    g_print("path: <%s>\n", path);
		    for (i=0; i<argc; i++) {
				g_print("arg %d '%c' ", i, types[i]);
				lo_arg_pp(types[i], argv[i]);
				g_print("\n");
		    }
		    fflush(stdout);
		}
	} else {
		return -1;
	}
    return 0;
}

static void osc_server_error(int num, const gchar *msg, const gchar *path)
{
    g_print("liblo server error %d in path %s: %s\n", num, path, msg);
}

static void osc_server_init(jmd_interface_t *interface)
{
	/* Create new server */
	interface->st = lo_server_thread_new(NULL, osc_server_error );

	/* Add the methods */
    lo_server_thread_add_method(interface->st, "/mixer/channel_count_is", "i", osc_channel_count_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/prefader_count_is", "i", osc_prefader_count_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/postfader_count_is", "i", osc_postfader_count_handler,interface);

    lo_server_thread_add_method(interface->st, "/mixer/channel/label_is", "is", osc_channel_label_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/gain_is", "iif", osc_channel_send_gain_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/pan_is", "iif", osc_channel_send_pan_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/gain_l_is", "iif", osc_channel_send_gain_l_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/gain_r_is", "iif", osc_channel_send_gain_r_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/mute_is", "ii", osc_channel_mute_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/stereo_is", "ii", osc_channel_stereo_handler,interface);

    lo_server_thread_add_method(interface->st, "/mixer/output/label_is", "is", osc_output_label_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/output/gain_is", "if", osc_output_gain_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/output/mute_is", "ii", osc_output_mute_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/output/link_is", "ii", osc_output_link_handler,interface);

    lo_server_thread_add_method(interface->st, "/mixer/channel/label_now", "is", osc_set_return_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/gain_now", "iif", osc_set_return_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/pan_now", "iif", osc_set_return_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/gain_l_now", "iif", osc_set_return_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/send/gain_r_now", "iif", osc_set_return_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/mute_now", "ii", osc_set_return_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/channel/stereo_now", "ii", osc_set_return_handler,interface);

    lo_server_thread_add_method(interface->st, "/mixer/output/label_now", "is", osc_set_return_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/output/gain_now", "if", osc_set_return_handler, interface);
    lo_server_thread_add_method(interface->st, "/mixer/output/mute_now", "ii", osc_set_return_handler,interface);
    lo_server_thread_add_method(interface->st, "/mixer/output/link_now", "ii", osc_set_return_handler,interface);

    lo_server_thread_add_method(interface->st, "/pong", "", osc_pong_handler,interface);

    /* add method that will match any path and args got to be last*/
    lo_server_thread_add_method(interface->st, NULL, NULL, osc_generic_handler,interface);

    lo_server_thread_start(interface->st);

    interface->t = lo_address_new(interface->osc_mixer_host, interface->osc_mixer_port);
	interface->serv = lo_server_thread_get_server( interface->st );
}

static void osc_server_finish(jmd_interface_t *interface)
{
	lo_server_thread_stop(interface->st);
	lo_server_thread_free(interface->st);

	if (interface->verbose) g_print( "Stopped OSC server thread.\n");
}

static void jack_shutdown_callback(void *arg)
{
	running = 0;
}

static gint jack_init_client(jmd_interface_t *interface)
{
	jack_status_t status;

	g_print("Connecting to Jack...\n");

	/* Register with Jack */
	if ((interface->jack_client = jack_client_open(interface->client_name, JackNullOption, &status)) == 0) {
		fprintf(stderr, "Failed to start jack client: %d\n", status);
		return 1;
	}

	/* Register shutdown callback */
	jack_on_shutdown (interface->jack_client, jack_shutdown_callback, interface);

	/* Register the peak audio callback */
	/*jack_set_process_callback(interface->jack_client, jack_process_audio, interface);*/

	g_print("Done\n");

	return 0;
}

static gint lash_init_client(int argc, gchar *argv[], jmd_interface_t *interface)
{
	gchar lowercase_name[strlen(interface->client_name)+1];
	gint i;
	lash_event_t *event;

	for (i=0;i<=strlen(interface->client_name);i++){
		lowercase_name[i]=tolower(interface->client_name[i]);
	}

	g_print("Connecting to LASH...\n");

	interface->lash_client = lash_init(lash_extract_args(&argc, &argv), lowercase_name,
					   0,LASH_PROTOCOL(2, 0));

	if (!interface->lash_client) {
		fprintf(stderr, "%s: could not initialize lash\n", __FUNCTION__);
		return 1;
	}

	event = lash_event_new_with_type(LASH_Client_Name);
	lash_event_set_string(event, interface->client_name);
	lash_send_event(interface->lash_client, event);

	g_print("Done\n");

	return 0;
}

static gint lash_jack_client(jmd_interface_t *interface)
{
	lash_event_t *event;

	event = lash_event_new_with_type(LASH_Jack_Client_Name);
	lash_event_set_string(event, interface->client_name);
	lash_send_event(interface->lash_client, event);

	return 0;
}

void *lash_process_events(void *ptr){

	lash_event_t *event=NULL;
	jmd_interface_t *interface = (jmd_interface_t*)ptr;
	while(running){
		event = lash_get_event(interface->lash_client);
		if (event) {
			if (interface->verbose) g_print("%s: got event of type %d with string '%s'\n",
				   __FUNCTION__, lash_event_get_type(event),
				   lash_event_get_string(event));
			switch (lash_event_get_type(event)) {
			case LASH_Restore_File:
			{
				sprintf(interface->xml_config_path,"%s",lash_event_get_string(event));
				lash_send_event(interface->lash_client, event);
				break;
			}
			case LASH_Save_File:
			{
				sprintf(interface->xml_config_path,"%s",lash_event_get_string(event));

				/* Retrieve port connections */
				if (interface->verbose) g_print("Calling get connections\n");
/*				if (jack_get_connects(interface)) {
					fprintf(stderr, "Error getting connections.\n");
					exit(1);
				}*/
				if (interface->verbose) g_print("Finished get connections\n");

				/* Create the config file */
				if (interface->verbose) g_print("Calling write config\n");
/*				if (xml_write_config(interface)) {
					fprintf(stderr, "Error writing config.\n");
					exit(1);
				}*/
				if (interface->verbose) g_print("Finished write config\n");

				lash_send_event(interface->lash_client, event);
				break;
			}
			case LASH_Quit:
			{
				running=0;
				gtk_main_quit();
				g_print("Quitting JackMixDesk_gtk.\n");
				/* Takes too long to exit so this one. */
				exit(0);
				break;
			}
			case LASH_Server_Lost:
			{
				g_print("Server connection lost, lashd running?\n");
				break;
			}
			default:
				fprintf(stderr,
						"%s: recieved unknown LASH event of type %d\n",
						__FUNCTION__, lash_event_get_type(event));
				lash_event_destroy(event);
				break;
			}
		}
		sleep(1);
	}
	pthread_exit(0);
}

void scrollbar_cb(GtkAdjustment *adj, gpointer user_data)
{
 g_print("%s hat sich verändert ",(char *)user_data);
 g_print("neue Pos. : %f\n", adj->value);
}

int gtk_test()
{
 GtkWidget *fenster;
 GtkWidget *box;
 GtkWidget *hscroll;
 GtkObject *adj;


 /*Ein neues Fenster erstellen*/
 fenster = gtk_window_new(GTK_WINDOW_TOPLEVEL);

 /*Neue Box erstellen*/
 box = gtk_hbox_new(FALSE,10);

 adj = gtk_adjustment_new(1.0, 1.0, 101.0, 1.0, 1.0, 1.0);

 hscroll = gtk_hscrollbar_new(GTK_ADJUSTMENT(adj));

 /*Fensterposition*/
 gtk_window_set_position(GTK_WINDOW(fenster),GTK_WIN_POS_CENTER);

 gtk_container_set_border_width(GTK_CONTAINER(fenster),25);

	gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                    GTK_SIGNAL_FUNC(scrollbar_cb), "Scrollbar");

 /*Nun alles rein in die Box....*/
 gtk_box_pack_start(GTK_BOX(box), hscroll, TRUE, TRUE, 0);


 gtk_container_add(GTK_CONTAINER(fenster),box);

 /*Zeigs uns.....*/
 gtk_widget_show_all(fenster);

 return 0;
}

int main(gint argc, gchar *argv[])
{
	gint opt;
	gint opt_ind=0;		/* getopt_long option index */

	pthread_t lash_thread;

	/* Set signal handlers */
	signal(SIGTERM, interface_signal_handler);
	signal(SIGINT, interface_signal_handler);
    /*
     * this initializes the xml2 library and checks potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
	LIBXML_TEST_VERSION

	jmd_interface_t *interface = NULL;

    gtk_init (&argc, &argv);

    interface=interface_init();


	while ((opt = getopt_long(argc, argv, "n:p:h:d:f:l:s:i:gcvq",opts_long, &opt_ind)) != -1) {
		switch (opt) {
			case 'n':  snprintf( interface->client_name, DEFAULT_LABEL_LEN, "%s", optarg ); break;
			case 'v':  interface->verbose=1; break;
			case 'q':  quiet=1; break;
			case 'd':  sprintf(interface->xml_config_path, "%s", optarg ); break;
			case 'f':  sprintf(interface->xml_config_file, "%s", optarg ); break;
			case 'a':  interface->omit_send_pan=1; break;
			case 'g':  interface->omit_send_gain=1; break;
			case 'c':  interface->omit_chan_pan=1; break;
			case 'p':  interface->osc_mixer_port = optarg; break;
			case 'h':  interface->osc_mixer_host = optarg; break;
			case 'l':  sprintf(interface->lash_project,"%s",optarg); break;
			case 's':  sprintf(interface->lash_server,"%s",optarg); break;
			case 'i':  sprintf(interface->lash_id,"%s",optarg); break;
			default:
				fprintf(stderr, "Unknown option '%c'.\n", (gchar)opt);
				interface_usage( argv[0] );
				break;
		}
	}

	/* Setup LASH client */
	if (!lash_init_client(argc,argv,interface)) {
		/* Start lash event thread */
		pthread_create( &lash_thread, NULL,lash_process_events , interface );
	} else {
		g_print("LASH support disabled.\n");
		gtk_message("LASH support disabled.",interface);
	}

	/* LASH nedds args unchanged */
	argc -= optind;
    argv += optind;

	osc_server_init(interface);

	interface_mixer_alive(interface);

	if(interface->alive){
    	interface_init_mixer(interface);
	}

	/* Setup JACK client */
	if (jack_init_client(interface)) {
		fprintf(stderr, "Error initializing jack client.\n");
		gtk_message("Error initializing jack client. Exiting!",interface);
		exit(1);
	}

	/* Send Jack client name to LASH */
	if (lash_jack_client(interface)) {
		fprintf(stderr, "Error sending Jack name to LASH.\n");
		gtk_message("Error sending Jack name to LASH.",interface);
		exit(1);
	}

	interface_welcome(interface);

    if(interface->alive){
    	interface_init_outputs_struct(interface);

    	interface_init_channels_struct(interface);

    	interface_get_outputs_data(interface);

    	interface_get_channels_data(interface);

    	/* Wait until OSC responses are processed */
    	usleep(500);
    }

	interface_init_style(interface);

    interface_init_gtk_menu(interface);

    if(interface->alive){
    	interface_init_gtk_outputs(interface);

    	interface_init_gtk_channels(interface);

    	interface_update_gtk_outputs(interface);

    	interface_update_gtk_channels(interface);
    }

    gtk_main ();

    osc_server_finish(interface);

    interface_finish(interface);

    g_print("Exiting JackMixDesk_gtk.\n");

    return 0;
}
